@@ -2,9 +2,9 @@
 /*! \file bitvectors.cpp
  ** \verbatim
  ** Top contributors (to current version):
- **   Liana Hadarean, Morgan Deters, Paul Meng
+ **   Liana Hadarean, Aina Niemetz, Morgan Deters
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -107,5 +107,21 @@ int main() {
   cout << " Expect valid. " << endl;
   cout << " CVC4: " << smt.query(new_x_eq_new_x_) << endl;
 
+  Expr x_neq_x = em.mkExpr(kind::EQUAL, x, x).notExpr();
+  std::vector<Expr> v{new_x_eq_new_x_, x_neq_x};
+  cout << " Querying: " << v << endl;
+  cout << " Expect invalid. " << endl;
+  cout << " CVC4: " << smt.query(v) << endl;
+
+  // Assert that a is odd 
+  Expr extract_op = em.mkConst(BitVectorExtract(0, 0));
+  Expr  lsb_of_a = em.mkExpr(extract_op, a);
+  cout << "Type of " << lsb_of_a << " is " << lsb_of_a.getType() << endl;
+  Expr a_odd = em.mkExpr(kind::EQUAL, lsb_of_a, em.mkConst(BitVector(1u, 1u)));
+  cout << "Assert " << a_odd << endl;
+  cout << "Check satisfiability." << endl;
+  smt.assertFormula(a_odd);
+  cout << " Expect sat. " << endl;
+  cout << " CVC4: " << smt.checkSat() << endl;
   return 0;
 }
