@@ -2,9 +2,9 @@
 /*! \file cdtrail_hashmap.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Tim King, Morgan Deters
+ **   Tim King, Mathias Preiner, Morgan Deters
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -44,9 +44,9 @@
 
 #pragma once
 
-#include <boost/static_assert.hpp>
-#include <ext/hash_map>
 #include <deque>
+#include <functional>
+#include <unordered_map>
 #include <utility>
 
 #include "base/cvc4_assert.h"
@@ -59,7 +59,7 @@ namespace CVC4 {
 namespace context {
 
 
-template <class Key, class Data, class HashFcn = __gnu_cxx::hash<Key> >
+template <class Key, class Data, class HashFcn = std::hash<Key> >
 class TrailHashMap {
 public:
   /** A pair of Key and Data that mirrors hash_map::value_type. */
@@ -92,8 +92,7 @@ private:
   /** The trail of elements. */
   KDTVec d_kdts;
 
-
-  typedef __gnu_cxx::hash_map<Key, size_t, HashFcn> PositionMap;
+  typedef std::unordered_map<Key, size_t, HashFcn> PositionMap;
   typedef typename PositionMap::iterator PM_iterator;
   typedef typename PositionMap::const_iterator PM_const_iterator;
 
@@ -124,13 +123,15 @@ private:
   }
 
 public:
-  /**
-   * Constant iterator for TrailHashMap.
-   * Only supports forward iteration.
-   * This always points at the end or a current element in the trail.
-   * This is done by iterating over the trail.
-   */
-  class const_iterator {
+ TrailHashMap() : d_kdts{}, d_posMap{}, d_uniqueKeys(0) {}
+
+ /**
+  * Constant iterator for TrailHashMap.
+  * Only supports forward iteration.
+  * This always points at the end or a current element in the trail.
+  * This is done by iterating over the trail.
+  */
+ class const_iterator {
   private:
     /** A vector iterator. */
     KDTVec_const_iterator d_it;
@@ -393,7 +394,8 @@ private:
    * the current sizes to a copy using the copy constructor,
    * The saved information is allocated using the ContextMemoryManager.
    */
-  ContextObj* save(ContextMemoryManager* pCMM) {
+  ContextObj* save(ContextMemoryManager* pCMM) override
+  {
     ContextObj* data = new(pCMM) CDTrailHashMap<Key, Data, HashFcn>(*this);
     Debug("CDTrailHashMap") << "save " << this
                             << " at level " << this->getContext()->getLevel()
@@ -408,20 +410,23 @@ protected:
    * restores the previous size.  Note that the list pointer and the
    * allocated size are not changed.
    */
-  void restore(ContextObj* data) {
-    Debug("CDTrailHashMap") << "restore " << this
-                            << " level " << this->getContext()->getLevel()
-                            << " data == " << data
-                            << " d_trailMap == " << this->d_trailMap << std::endl;
-    size_t oldSize = ((CDTrailHashMap<Key, Data, HashFcn>*)data)->d_trailSize;
-    d_trailMap->pop_to_size(oldSize);
-    d_trailSize = oldSize;
-    Assert(d_trailMap->trailSize() == d_trailSize);
+ void restore(ContextObj* data) override
+ {
+   Debug("CDTrailHashMap") << "restore " << this << " level "
+                           << this->getContext()->getLevel()
+                           << " data == " << data
+                           << " d_trailMap == " << this->d_trailMap
+                           << std::endl;
+   size_t oldSize = ((CDTrailHashMap<Key, Data, HashFcn>*)data)->d_trailSize;
+   d_trailMap->pop_to_size(oldSize);
+   d_trailSize = oldSize;
+   Assert(d_trailMap->trailSize() == d_trailSize);
 
-    d_prevTrailSize = ((CDTrailHashMap<Key, Data, HashFcn>*)data)->d_prevTrailSize;
-    Debug("CDTrailHashMap") << "restore " << this
-                            << " level " << this->getContext()->getLevel()
-                            << " size back to " << this->d_trailSize << std::endl;
+   d_prevTrailSize =
+       ((CDTrailHashMap<Key, Data, HashFcn>*)data)->d_prevTrailSize;
+   Debug("CDTrailHashMap") << "restore " << this << " level "
+                           << this->getContext()->getLevel() << " size back to "
+                           << this->d_trailSize << std::endl;
   }
 
   /**
@@ -551,7 +556,7 @@ public:
 };/* class CDTrailHashMap<> */
 
 template <class Data, class HashFcn>
-class CDTrailHashMap <TNode, Data, HashFcn > : public ContextObj {
+class CDTrailHashMap<TNode, Data, HashFcn> : public ContextObj {
   /* CDTrailHashMap is challenging to get working with TNode.
    * Consider using CDHashMap<TNode,...> instead.
    *
@@ -563,7 +568,8 @@ class CDTrailHashMap <TNode, Data, HashFcn > : public ContextObj {
    * hashed. Getting the order right with a guarantee is too hard.
    */
 
-  BOOST_STATIC_ASSERT(sizeof(Data) == 0);
+  static_assert(sizeof(Data) == 0,
+                "Cannot create a CDTrailHashMap with TNode keys");
 };
 
 }/* CVC4::context namespace */
