@@ -2,9 +2,9 @@
 /*! \file justification_heuristic.cpp
  ** \verbatim
  ** Top contributors (to current version):
- **   Kshitij Bansal, Tim King, Morgan Deters
+ **   Kshitij Bansal, Aina Niemetz, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -24,6 +24,7 @@
 #include "theory/rewriter.h"
 #include "smt/term_formula_removal.h"
 #include "smt/smt_statistics_registry.h"
+#include "util/random.h"
 
 namespace CVC4 {
 
@@ -54,7 +55,8 @@ JustificationHeuristic::JustificationHeuristic(CVC4::DecisionEngine* de,
   Trace("decision") << "Justification heuristic enabled" << std::endl;
 }
 
-JustificationHeuristic::~JustificationHeuristic() throw() {
+JustificationHeuristic::~JustificationHeuristic()
+{
   smtStatisticsRegistry()->unregisterStat(&d_helfulness);
   smtStatisticsRegistry()->unregisterStat(&d_giveup);
   smtStatisticsRegistry()->unregisterStat(&d_timestat);
@@ -298,33 +300,35 @@ DecisionWeight JustificationHeuristic::getWeightPolarized(TNode n, bool polarity
 DecisionWeight JustificationHeuristic::getWeight(TNode n) {
   if(!n.hasAttribute(DecisionWeightAttr()) ) {
 
-    DecisionWeightInternal combiningFn =
-      options::decisionWeightInternal();
+    DecisionWeightInternal combiningFn = options::decisionWeightInternal();
 
-    if(combiningFn == DECISION_WEIGHT_INTERNAL_OFF || n.getNumChildren() == 0) {
-
-      if(options::decisionRandomWeight() != 0) {
-        n.setAttribute(DecisionWeightAttr(), rand() % options::decisionRandomWeight());
+    if (combiningFn == DECISION_WEIGHT_INTERNAL_OFF || n.getNumChildren() == 0)
+    {
+      if (options::decisionRandomWeight() != 0)
+      {
+        n.setAttribute(DecisionWeightAttr(),
+            Random::getRandom().pick(0, options::decisionRandomWeight()-1));
       }
-
-    } else if(combiningFn == DECISION_WEIGHT_INTERNAL_MAX) {
-
+    }
+    else if (combiningFn == DECISION_WEIGHT_INTERNAL_MAX)
+    {
       DecisionWeight dW = 0;
-      for(TNode::iterator i=n.begin(); i != n.end(); ++i)
+      for (TNode::iterator i = n.begin(); i != n.end(); ++i)
         dW = max(dW, getWeight(*i));
       n.setAttribute(DecisionWeightAttr(), dW);
-
-    } else if(combiningFn == DECISION_WEIGHT_INTERNAL_SUM ||
-              combiningFn == DECISION_WEIGHT_INTERNAL_USR1) {
+    }
+    else if (combiningFn == DECISION_WEIGHT_INTERNAL_SUM
+             || combiningFn == DECISION_WEIGHT_INTERNAL_USR1)
+    {
       DecisionWeight dW = 0;
-      for(TNode::iterator i=n.begin(); i != n.end(); ++i)
+      for (TNode::iterator i = n.begin(); i != n.end(); ++i)
         dW = max(dW, getWeight(*i));
       n.setAttribute(DecisionWeightAttr(), dW);
-
-    } else {
+    }
+    else
+    {
       Unreachable();
     }
-
   }
   return n.getAttribute(DecisionWeightAttr());
 }
