@@ -4,7 +4,7 @@
  ** Top contributors (to current version):
  **   Andrew Reynolds, Morgan Deters, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -28,6 +28,7 @@
 #include "expr/node_manager.h"
 #include "expr/type.h"
 #include "options/set_language.h"
+#include "options/datatypes_options.h"
 
 using namespace std;
 
@@ -76,6 +77,11 @@ size_t Datatype::indexOf(Expr item) {
                 item.getType().isSelector(),
                 item,
                 "arg must be a datatype constructor, selector, or tester");
+  return indexOfInternal(item);
+}
+
+size_t Datatype::indexOfInternal(Expr item)
+{
   TNode n = Node::fromExpr(item);
   if( item.getKind()==kind::APPLY_TYPE_ASCRIPTION ){
     return indexOf( item[0] );
@@ -90,6 +96,10 @@ size_t Datatype::cindexOf(Expr item) {
   PrettyCheckArgument(item.getType().isSelector(),
                 item,
                 "arg must be a datatype selector");
+  return cindexOfInternal(item);
+}
+size_t Datatype::cindexOfInternal(Expr item)
+{
   TNode n = Node::fromExpr(item);
   if( item.getKind()==kind::APPLY_TYPE_ASCRIPTION ){
     return cindexOf( item[0] );
@@ -105,8 +115,7 @@ void Datatype::resolve(ExprManager* em,
                        const std::vector<Type>& replacements,
                        const std::vector< SortConstructorType >& paramTypes,
                        const std::vector< DatatypeType >& paramReplacements)
-  throw(IllegalArgumentException, DatatypeResolutionException) {
-
+{
   PrettyCheckArgument(em != NULL, em, "cannot resolve a Datatype with a NULL expression manager");
   PrettyCheckArgument(!d_resolved, this, "cannot resolve a Datatype twice");
   PrettyCheckArgument(resolutions.find(d_name) != resolutions.end(), resolutions,
@@ -161,7 +170,7 @@ void Datatype::resolve(ExprManager* em,
     evalType.push_back(TypeNode::fromType(d_sygus_type));
     TypeNode eval_func_type = nm->mkFunctionType(evalType);
     d_sygus_eval = nm->mkSkolem(name, eval_func_type, "sygus evaluation function").toExpr();    
-  }  
+  }
 }
 
 void Datatype::addConstructor(const DatatypeConstructor& c) {
@@ -180,6 +189,29 @@ void Datatype::setSygus( Type st, Expr bvl, bool allow_const, bool allow_all ){
   d_sygus_allow_all = allow_all;
 }
 
+void Datatype::addSygusConstructor(Expr op,
+                                   std::string& cname,
+                                   std::vector<Type>& cargs,
+                                   std::shared_ptr<SygusPrintCallback> spc,
+                                   int weight)
+{
+  Debug("dt-sygus") << "--> Add constructor " << cname << " to " << getName() << std::endl;
+  Debug("dt-sygus") << "    sygus op : " << op << std::endl;
+  std::string name = getName() + "_" + cname;
+  std::string testerId("is-");
+  testerId.append(name);
+  unsigned cweight = weight >= 0 ? weight : (cargs.empty() ? 0 : 1);
+  DatatypeConstructor c(name, testerId, cweight);
+  c.setSygus(op, spc);
+  for( unsigned j=0; j<cargs.size(); j++ ){
+    Debug("parser-sygus-debug") << "  arg " << j << " : " << cargs[j] << std::endl;
+    std::stringstream sname;
+    sname << name << "_" << j;
+    c.addArg(sname.str(), cargs[j]);
+  }
+  addConstructor(c);
+}
+                                    
 void Datatype::setTuple() {
   PrettyCheckArgument(!d_resolved, this, "cannot set tuple to a finalized Datatype");
   d_isTuple = true;
@@ -190,7 +222,8 @@ void Datatype::setRecord() {
   d_isRecord = true;
 }
 
-Cardinality Datatype::getCardinality( Type t ) const throw(IllegalArgumentException) {
+Cardinality Datatype::getCardinality(Type t) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
   Assert( t.isDatatype() && ((DatatypeType)t).getDatatype()==*this );
   std::vector< Type > processing;
@@ -198,12 +231,15 @@ Cardinality Datatype::getCardinality( Type t ) const throw(IllegalArgumentExcept
   return d_card;
 }
 
-Cardinality Datatype::getCardinality() const throw(IllegalArgumentException) {
+Cardinality Datatype::getCardinality() const
+{
   PrettyCheckArgument(!isParametric(), this, "for getCardinality, this datatype cannot be parametric");
   return getCardinality( d_self );
 }
 
-Cardinality Datatype::computeCardinality( Type t, std::vector< Type >& processing ) const throw(IllegalArgumentException){
+Cardinality Datatype::computeCardinality(Type t,
+                                         std::vector<Type>& processing) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
   if( std::find( processing.begin(), processing.end(), d_self )!=processing.end() ){
     d_card = Cardinality::INTEGERS;
@@ -219,7 +255,8 @@ Cardinality Datatype::computeCardinality( Type t, std::vector< Type >& processin
   return d_card;
 }
 
-bool Datatype::isRecursiveSingleton( Type t ) const throw(IllegalArgumentException) {
+bool Datatype::isRecursiveSingleton(Type t) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
   Assert( t.isDatatype() && ((DatatypeType)t).getDatatype()==*this );
   if( d_card_rec_singleton.find( t )==d_card_rec_singleton.end() ){
@@ -245,34 +282,42 @@ bool Datatype::isRecursiveSingleton( Type t ) const throw(IllegalArgumentExcepti
   return d_card_rec_singleton[t]==1;
 }
 
-bool Datatype::isRecursiveSingleton() const throw(IllegalArgumentException) {
+bool Datatype::isRecursiveSingleton() const
+{
   PrettyCheckArgument(!isParametric(), this, "for isRecursiveSingleton, this datatype cannot be parametric");
   return isRecursiveSingleton( d_self );
 }
 
-unsigned Datatype::getNumRecursiveSingletonArgTypes( Type t ) const throw(IllegalArgumentException) {
+unsigned Datatype::getNumRecursiveSingletonArgTypes(Type t) const
+{
   Assert( d_card_rec_singleton.find( t )!=d_card_rec_singleton.end() );
   Assert( isRecursiveSingleton( t ) );
   return d_card_u_assume[t].size();
 }
 
-unsigned Datatype::getNumRecursiveSingletonArgTypes() const throw(IllegalArgumentException) {
+unsigned Datatype::getNumRecursiveSingletonArgTypes() const
+{
   PrettyCheckArgument(!isParametric(), this, "for getNumRecursiveSingletonArgTypes, this datatype cannot be parametric");
   return getNumRecursiveSingletonArgTypes( d_self );
 }
 
-Type Datatype::getRecursiveSingletonArgType( Type t, unsigned i ) const throw(IllegalArgumentException) {
+Type Datatype::getRecursiveSingletonArgType(Type t, unsigned i) const
+{
   Assert( d_card_rec_singleton.find( t )!=d_card_rec_singleton.end() );
   Assert( isRecursiveSingleton( t ) );
   return d_card_u_assume[t][i];
 }
 
-Type Datatype::getRecursiveSingletonArgType( unsigned i ) const throw(IllegalArgumentException) {
+Type Datatype::getRecursiveSingletonArgType(unsigned i) const
+{
   PrettyCheckArgument(!isParametric(), this, "for getRecursiveSingletonArgType, this datatype cannot be parametric");
   return getRecursiveSingletonArgType( d_self, i );
 }
 
-bool Datatype::computeCardinalityRecSingleton( Type t, std::vector< Type >& processing, std::vector< Type >& u_assume ) const throw(IllegalArgumentException){
+bool Datatype::computeCardinalityRecSingleton(Type t,
+                                              std::vector<Type>& processing,
+                                              std::vector<Type>& u_assume) const
+{
   if( std::find( processing.begin(), processing.end(), d_self )!=processing.end() ){
     return true;
   }else{
@@ -319,7 +364,8 @@ bool Datatype::computeCardinalityRecSingleton( Type t, std::vector< Type >& proc
   }
 }
 
-bool Datatype::isFinite( Type t ) const throw(IllegalArgumentException) {
+bool Datatype::isFinite(Type t) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
   Assert( t.isDatatype() && ((DatatypeType)t).getDatatype()==*this );
 
@@ -341,12 +387,14 @@ bool Datatype::isFinite( Type t ) const throw(IllegalArgumentException) {
   self.setAttribute(DatatypeFiniteAttr(), true);
   return true;
 }
-bool Datatype::isFinite() const throw(IllegalArgumentException) {
+bool Datatype::isFinite() const
+{
   PrettyCheckArgument(isResolved() && !isParametric(), this, "this datatype must be resolved and not parametric");
   return isFinite( d_self );
 }
 
-bool Datatype::isInterpretedFinite( Type t ) const throw(IllegalArgumentException) {
+bool Datatype::isInterpretedFinite(Type t) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
   Assert( t.isDatatype() && ((DatatypeType)t).getDatatype()==*this );
   // we're using some internals, so we have to set up this library context
@@ -368,12 +416,14 @@ bool Datatype::isInterpretedFinite( Type t ) const throw(IllegalArgumentExceptio
   self.setAttribute(DatatypeUFiniteAttr(), true);
   return true;
 }
-bool Datatype::isInterpretedFinite() const throw(IllegalArgumentException) {
+bool Datatype::isInterpretedFinite() const
+{
   PrettyCheckArgument(isResolved() && !isParametric(), this, "this datatype must be resolved and not parametric");
   return isInterpretedFinite( d_self );
 }
 
-bool Datatype::isWellFounded() const throw(IllegalArgumentException) {
+bool Datatype::isWellFounded() const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
   if( d_well_founded==0 ){
     // we're using some internals, so we have to set up this library context
@@ -388,7 +438,8 @@ bool Datatype::isWellFounded() const throw(IllegalArgumentException) {
   return d_well_founded==1;
 }
 
-bool Datatype::computeWellFounded( std::vector< Type >& processing ) const throw(IllegalArgumentException) {
+bool Datatype::computeWellFounded(std::vector<Type>& processing) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
   if( std::find( processing.begin(), processing.end(), d_self )!=processing.end() ){
     return d_isCo;
@@ -408,7 +459,8 @@ bool Datatype::computeWellFounded( std::vector< Type >& processing ) const throw
   }
 }
 
-Expr Datatype::mkGroundTerm( Type t ) const throw(IllegalArgumentException) {
+Expr Datatype::mkGroundTerm(Type t) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
   ExprManagerScope ems(d_self);
   Debug("datatypes") << "mkGroundTerm of type " << t << std::endl;
@@ -452,7 +504,8 @@ Expr getSubtermWithType( Expr e, Type t, bool isTop ){
   }
 }
 
-Expr Datatype::computeGroundTerm( Type t, std::vector< Type >& processing ) const throw(IllegalArgumentException) {
+Expr Datatype::computeGroundTerm(Type t, std::vector<Type>& processing) const
+{
   if( std::find( processing.begin(), processing.end(), t )==processing.end() ){
     processing.push_back( t );
     for( unsigned r=0; r<2; r++ ){
@@ -483,20 +536,22 @@ Expr Datatype::computeGroundTerm( Type t, std::vector< Type >& processing ) cons
   return Expr();
 }
 
-DatatypeType Datatype::getDatatypeType() const throw(IllegalArgumentException) {
+DatatypeType Datatype::getDatatypeType() const
+{
   PrettyCheckArgument(isResolved(), *this, "Datatype must be resolved to get its DatatypeType");
   PrettyCheckArgument(!d_self.isNull(), *this);
   return DatatypeType(d_self);
 }
 
-DatatypeType Datatype::getDatatypeType(const std::vector<Type>& params)
-  const throw(IllegalArgumentException) {
+DatatypeType Datatype::getDatatypeType(const std::vector<Type>& params) const
+{
   PrettyCheckArgument(isResolved(), *this, "Datatype must be resolved to get its DatatypeType");
   PrettyCheckArgument(!d_self.isNull() && DatatypeType(d_self).isParametric(), this);
   return DatatypeType(d_self).instantiate(params);
 }
 
-bool Datatype::operator==(const Datatype& other) const throw() {
+bool Datatype::operator==(const Datatype& other) const
+{
   // two datatypes are == iff the name is the same and they have
   // exactly matching constructors (in the same order)
 
@@ -584,6 +639,30 @@ const DatatypeConstructor& Datatype::operator[](std::string name) const {
   IllegalArgument(name, "No such constructor `%s' of datatype `%s'", name.c_str(), d_name.c_str());
 }
 
+
+Expr Datatype::getSharedSelector( Type dtt, Type t, unsigned index ) const{
+  PrettyCheckArgument(isResolved(), this, "this datatype is not yet resolved");
+  std::map< Type, std::map< Type, std::map< unsigned, Expr > > >::iterator itd = d_shared_sel.find( dtt );
+  if( itd!=d_shared_sel.end() ){
+    std::map< Type, std::map< unsigned, Expr > >::iterator its = itd->second.find( t );
+    if( its!=itd->second.end() ){
+      std::map< unsigned, Expr >::iterator it = its->second.find( index );
+      if( it!=its->second.end() ){
+        return it->second;
+      }
+    }
+  }
+  //make the shared selector
+  Expr s;
+  NodeManager* nm = NodeManager::fromExprManager( d_self.getExprManager() );
+  std::stringstream ss;
+  ss << "sel_" << index;
+  s = nm->mkSkolem(ss.str(), nm->mkSelectorType(TypeNode::fromType(dtt), TypeNode::fromType(t)), "is a shared selector", NodeManager::SKOLEM_NO_NOTIFY).toExpr();
+  d_shared_sel[dtt][t][index] = s;
+  Trace("dt-shared-sel") << "Made " << s << " of type " << dtt << " -> " << t << std::endl;
+  return s; 
+}
+
 Expr Datatype::getConstructor(std::string name) const {
   return (*this)[name].getConstructor();
 }
@@ -622,8 +701,7 @@ void DatatypeConstructor::resolve(ExprManager* em, DatatypeType self,
                                   const std::vector<Type>& replacements,
                                   const std::vector< SortConstructorType >& paramTypes,
                                   const std::vector< DatatypeType >& paramReplacements, size_t cindex)
-  throw(IllegalArgumentException, DatatypeResolutionException) {
-
+{
   PrettyCheckArgument(em != NULL, em, "cannot resolve a Datatype with a NULL expression manager");
   PrettyCheckArgument(!isResolved(),
                 "cannot resolve a Datatype constructor twice; "
@@ -716,34 +794,44 @@ Type DatatypeConstructor::doParametricSubstitution( Type range,
   }
 }
 
-DatatypeConstructor::DatatypeConstructor(std::string name) :
-  // We don't want to introduce a new data member, because eventually
-  // we're going to be a constant stuffed inside a node.  So we stow
-  // the tester name away inside the constructor name until
-  // resolution.
-  d_name(name + '\0' + "is_" + name), // default tester name is "is_FOO"
-  d_tester(),
-  d_args() {
+DatatypeConstructor::DatatypeConstructor(std::string name)
+    :  // We don't want to introduce a new data member, because eventually
+       // we're going to be a constant stuffed inside a node.  So we stow
+       // the tester name away inside the constructor name until
+       // resolution.
+      d_name(name + '\0' + "is_" + name),  // default tester name is "is_FOO"
+      d_tester(),
+      d_args(),
+      d_sygus_pc(nullptr),
+      d_weight(1)
+{
   PrettyCheckArgument(name != "", name, "cannot construct a datatype constructor without a name");
 }
 
-DatatypeConstructor::DatatypeConstructor(std::string name, std::string tester) :
-  // We don't want to introduce a new data member, because eventually
-  // we're going to be a constant stuffed inside a node.  So we stow
-  // the tester name away inside the constructor name until
-  // resolution.
-  d_name(name + '\0' + tester),
-  d_tester(),
-  d_args() {
+DatatypeConstructor::DatatypeConstructor(std::string name,
+                                         std::string tester,
+                                         unsigned weight)
+    :  // We don't want to introduce a new data member, because eventually
+       // we're going to be a constant stuffed inside a node.  So we stow
+       // the tester name away inside the constructor name until
+       // resolution.
+      d_name(name + '\0' + tester),
+      d_tester(),
+      d_args(),
+      d_sygus_pc(nullptr),
+      d_weight(weight)
+{
   PrettyCheckArgument(name != "", name, "cannot construct a datatype constructor without a name");
   PrettyCheckArgument(!tester.empty(), tester, "cannot construct a datatype constructor without a tester");
 }
 
-void DatatypeConstructor::setSygus( Expr op, Expr let_body, std::vector< Expr >& let_args, unsigned num_let_input_args ){
+void DatatypeConstructor::setSygus(Expr op,
+                                   std::shared_ptr<SygusPrintCallback> spc)
+{
+  PrettyCheckArgument(
+      !isResolved(), this, "cannot modify a finalized Datatype constructor");
   d_sygus_op = op;
-  d_sygus_let_body = let_body;
-  d_sygus_let_args.insert( d_sygus_let_args.end(), let_args.begin(), let_args.end() );
-  d_sygus_num_let_input_args = num_let_input_args;
+  d_sygus_pc = spc;
 }
 
 void DatatypeConstructor::addArg(std::string selectorName, Type selectorType) {
@@ -782,11 +870,13 @@ void DatatypeConstructor::addArg(std::string selectorName, DatatypeSelfType) {
   d_args.push_back(DatatypeConstructorArg(selectorName + '\0', Expr()));
 }
 
-std::string DatatypeConstructor::getName() const throw() {
+std::string DatatypeConstructor::getName() const
+{
   return d_name.substr(0, d_name.find('\0'));
 }
 
-std::string DatatypeConstructor::getTesterName() const throw() {
+std::string DatatypeConstructor::getTesterName() const
+{
   return d_name.substr(d_name.find('\0') + 1);
 }
 
@@ -797,6 +887,7 @@ Expr DatatypeConstructor::getConstructor() const {
 
 Type DatatypeConstructor::getSpecializedConstructorType(Type returnType) const {
   PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
+  PrettyCheckArgument(returnType.isDatatype(), this, "cannot get specialized constructor type for non-datatype type");
   ExprManagerScope ems(d_constructor);
   const Datatype& dt = Datatype::datatypeOf(d_constructor);
   PrettyCheckArgument(dt.isParametric(), this, "this datatype constructor is not parametric");
@@ -819,32 +910,29 @@ Expr DatatypeConstructor::getSygusOp() const {
   return d_sygus_op;
 }
 
-Expr DatatypeConstructor::getSygusLetBody() const {
-  PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
-  return d_sygus_let_body;
-}
-
-unsigned DatatypeConstructor::getNumSygusLetArgs() const {
-  PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
-  return d_sygus_let_args.size();
-}
-
-Expr DatatypeConstructor::getSygusLetArg( unsigned i ) const {
-  PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
-  return d_sygus_let_args[i];
-}
-
-unsigned DatatypeConstructor::getNumSygusLetInputArgs() const {
-  PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
-  return d_sygus_num_let_input_args;
-}
-
 bool DatatypeConstructor::isSygusIdFunc() const {
   PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
-  return d_sygus_let_args.size()==1 && d_sygus_let_args[0]==d_sygus_let_body;
+  return (d_sygus_op.getKind() == kind::LAMBDA
+          && d_sygus_op[0].getNumChildren() == 1
+          && d_sygus_op[0][0] == d_sygus_op[1]);
 }
 
-Cardinality DatatypeConstructor::getCardinality( Type t ) const throw(IllegalArgumentException) {
+unsigned DatatypeConstructor::getWeight() const
+{
+  PrettyCheckArgument(
+      isResolved(), this, "this datatype constructor is not yet resolved");
+  return d_weight;
+}
+
+std::shared_ptr<SygusPrintCallback> DatatypeConstructor::getSygusPrintCallback() const
+{
+  PrettyCheckArgument(
+      isResolved(), this, "this datatype constructor is not yet resolved");
+  return d_sygus_pc;
+}
+
+Cardinality DatatypeConstructor::getCardinality(Type t) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
 
   Cardinality c = 1;
@@ -857,7 +945,9 @@ Cardinality DatatypeConstructor::getCardinality( Type t ) const throw(IllegalArg
 }
 
 /** compute the cardinality of this datatype */
-Cardinality DatatypeConstructor::computeCardinality( Type t, std::vector< Type >& processing ) const throw(IllegalArgumentException){
+Cardinality DatatypeConstructor::computeCardinality(
+    Type t, std::vector<Type>& processing) const
+{
   Cardinality c = 1;
   std::vector< Type > instTypes;
   std::vector< Type > paramTypes;
@@ -880,7 +970,9 @@ Cardinality DatatypeConstructor::computeCardinality( Type t, std::vector< Type >
   return c;
 }
 
-bool DatatypeConstructor::computeWellFounded( std::vector< Type >& processing ) const throw(IllegalArgumentException){
+bool DatatypeConstructor::computeWellFounded(
+    std::vector<Type>& processing) const
+{
   for(const_iterator i = begin(), i_end = end(); i != i_end; ++i) {
     Type t = SelectorType((*i).getSelector().getType()).getRangeType();
     if( t.isDatatype() ){
@@ -893,8 +985,8 @@ bool DatatypeConstructor::computeWellFounded( std::vector< Type >& processing )
   return true;
 }
 
-
-bool DatatypeConstructor::isFinite( Type t ) const throw(IllegalArgumentException) {
+bool DatatypeConstructor::isFinite(Type t) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
 
   // we're using some internals, so we have to set up this library context
@@ -926,7 +1018,8 @@ bool DatatypeConstructor::isFinite( Type t ) const throw(IllegalArgumentExceptio
   return true;
 }
 
-bool DatatypeConstructor::isInterpretedFinite( Type t ) const throw(IllegalArgumentException) {
+bool DatatypeConstructor::isInterpretedFinite(Type t) const
+{
   PrettyCheckArgument(isResolved(), this, "this datatype constructor is not yet resolved");
   // we're using some internals, so we have to set up this library context
   ExprManagerScope ems(d_constructor);
@@ -958,8 +1051,11 @@ bool DatatypeConstructor::isInterpretedFinite( Type t ) const throw(IllegalArgum
   return true;
 }
 
-Expr DatatypeConstructor::computeGroundTerm( Type t, std::vector< Type >& processing, std::map< Type, Expr >& gt ) const throw(IllegalArgumentException) {
-// we're using some internals, so we have to set up this library context
+Expr DatatypeConstructor::computeGroundTerm(Type t,
+                                            std::vector<Type>& processing,
+                                            std::map<Type, Expr>& gt) const
+{
+  // we're using some internals, so we have to set up this library context
   ExprManagerScope ems(d_constructor);
 
   std::vector<Expr> groundTerms;
@@ -1011,6 +1107,30 @@ Expr DatatypeConstructor::computeGroundTerm( Type t, std::vector< Type >& proces
   return groundTerm;
 }
 
+void DatatypeConstructor::computeSharedSelectors( Type domainType ) const {
+  if( d_shared_selectors[domainType].size()<getNumArgs() ){
+    TypeNode ctype;
+    if( DatatypeType(domainType).isParametric() ){
+      ctype = TypeNode::fromType( getSpecializedConstructorType( domainType ) );
+    }else{
+      ctype = TypeNode::fromType( d_constructor.getType() );
+    }
+    Assert( ctype.isConstructor() );
+    Assert( ctype.getNumChildren()-1==getNumArgs() );
+    //compute the shared selectors
+    const Datatype& dt = Datatype::datatypeOf(d_constructor);
+    std::map< TypeNode, unsigned > counter;
+    for( unsigned j=0; j<ctype.getNumChildren()-1; j++ ){
+      TypeNode t = ctype[j];
+      Expr ss = dt.getSharedSelector( domainType, t.toType(), counter[t] );
+      d_shared_selectors[domainType].push_back( ss );
+      Assert( d_shared_selector_index[domainType].find( ss )==d_shared_selector_index[domainType].end() );
+      d_shared_selector_index[domainType][ss] = j;
+      counter[t]++;
+    }
+  }
+}
+
 
 const DatatypeConstructorArg& DatatypeConstructor::operator[](size_t index) const {
   PrettyCheckArgument(index < getNumArgs(), index, "index out of bounds");
@@ -1030,6 +1150,12 @@ Expr DatatypeConstructor::getSelector(std::string name) const {
   return (*this)[name].getSelector();
 }
 
+Type DatatypeConstructor::getArgType(unsigned index) const
+{
+  PrettyCheckArgument(index < getNumArgs(), index, "index out of bounds");
+  return static_cast<SelectorType>((*this)[index].getType()).getRangeType();
+}
+
 bool DatatypeConstructor::involvesExternalType() const{
   for(const_iterator i = begin(); i != end(); ++i) {
     if(! SelectorType((*i).getSelector().getType()).getRangeType().isDatatype()) {
@@ -1055,7 +1181,8 @@ DatatypeConstructorArg::DatatypeConstructorArg(std::string name, Expr selector)
   PrettyCheckArgument(name != "", name, "cannot construct a datatype constructor arg without a name");
 }
 
-std::string DatatypeConstructorArg::getName() const throw() {
+std::string DatatypeConstructorArg::getName() const
+{
   string name = d_name;
   const size_t nul = name.find('\0');
   if(nul != string::npos) {
@@ -1069,6 +1196,37 @@ Expr DatatypeConstructorArg::getSelector() const {
   return d_selector;
 }
 
+Expr DatatypeConstructor::getSelectorInternal( Type domainType, size_t index ) const {
+  PrettyCheckArgument(isResolved(), this, "cannot get an internal selector for an unresolved datatype constructor");
+  PrettyCheckArgument(index < getNumArgs(), index, "index out of bounds");
+  if( options::dtSharedSelectors() ){
+    computeSharedSelectors( domainType );
+    Assert( d_shared_selectors[domainType].size()==getNumArgs() );
+    return d_shared_selectors[domainType][index];
+  }else{
+    return d_args[index].getSelector();
+  }
+}
+
+int DatatypeConstructor::getSelectorIndexInternal( Expr sel ) const {
+  PrettyCheckArgument(isResolved(), this, "cannot get an internal selector index for an unresolved datatype constructor");
+  if( options::dtSharedSelectors() ){
+    Assert( sel.getType().isSelector() );
+    Type domainType = ((SelectorType)sel.getType()).getDomain();
+    computeSharedSelectors( domainType );
+    std::map< Expr, unsigned >::iterator its = d_shared_selector_index[domainType].find( sel );
+    if( its!=d_shared_selector_index[domainType].end() ){
+      return (int)its->second;
+    }
+  }else{
+    unsigned sindex = Datatype::indexOf(sel);
+    if( getNumArgs() > sindex && d_args[sindex].getSelector() == sel ){
+      return (int)sindex;
+    }
+  }
+  return -1;
+}
+
 Expr DatatypeConstructorArg::getConstructor() const {
   PrettyCheckArgument(isResolved(), this,
                 "cannot get a associated constructor for argument of an unresolved datatype constructor");
@@ -1083,7 +1241,8 @@ Type DatatypeConstructorArg::getRangeType() const {
   return getType().getRangeType();
 }
 
-bool DatatypeConstructorArg::isUnresolvedSelf() const throw() {
+bool DatatypeConstructorArg::isUnresolvedSelf() const
+{
   return d_selector.isNull() && d_name.size() == d_name.find('\0') + 1;
 }
 
@@ -1191,10 +1350,7 @@ std::ostream& operator<<(std::ostream& os, const DatatypeConstructorArg& arg) {
   return os;
 }
 
-DatatypeIndexConstant::DatatypeIndexConstant(unsigned index) throw(IllegalArgumentException) : d_index(index){
-
-}
-
+DatatypeIndexConstant::DatatypeIndexConstant(unsigned index) : d_index(index) {}
 std::ostream& operator<<(std::ostream& out, const DatatypeIndexConstant& dic) {
   return out << "index_" << dic.getIndex();
 }
