@@ -4,7 +4,7 @@
  ** Top contributors (to current version):
  **   Morgan Deters, Dejan Jovanovic, Christopher L. Conway
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -155,10 +155,11 @@
 #ifndef __CVC4__NODE_BUILDER_H
 #define __CVC4__NODE_BUILDER_H
 
-#include <iostream>
-#include <vector>
 #include <cstdlib>
+#include <iostream>
+#include <memory>
 #include <stdint.h>
+#include <vector>
 
 namespace CVC4 {
   static const unsigned default_nchild_thresh = 10;
@@ -171,7 +172,6 @@ namespace CVC4 {
 
 #include "base/cvc4_assert.h"
 #include "base/output.h"
-#include "base/ptr_closer.h"
 #include "expr/kind.h"
 #include "expr/metakind.h"
 #include "expr/node_value.h"
@@ -179,12 +179,6 @@ namespace CVC4 {
 
 namespace CVC4 {
 
-/* see expr/convenience_node_builders.h */
-class AndNodeBuilder;
-class OrNodeBuilder;
-class PlusNodeBuilder;
-class MultNodeBuilder;
-
 // Sometimes it's useful for debugging to output a NodeBuilder that
 // isn't yet a Node..
 template <unsigned nchild_thresh>
@@ -693,10 +687,11 @@ private:
   expr::NodeValue* constructNV() const;
 
 #ifdef CVC4_DEBUG
-  void maybeCheckType(const TNode n) const
-    throw(TypeCheckingExceptionPrivate, AssertionException);
+  // Throws a TypeCheckingExceptionPrivate on a failure.
+  void maybeCheckType(const TNode n) const;
 #else /* CVC4_DEBUG */
-  inline void maybeCheckType(const TNode n) const throw() { }
+  // Do nothing if not in debug mode.
+  inline void maybeCheckType(const TNode n) const {}
 #endif /* CVC4_DEBUG */
 
 public:
@@ -728,11 +723,6 @@ public:
   NodeBuilder<nchild_thresh>& operator-=(TNode);
   NodeBuilder<nchild_thresh>& operator*=(TNode);
 
-  friend class AndNodeBuilder;
-  friend class OrNodeBuilder;
-  friend class PlusNodeBuilder;
-  friend class MultNodeBuilder;
-
   // This is needed for copy constructors of different sizes to access
   // private fields
   template <unsigned N>
@@ -890,14 +880,14 @@ template <unsigned nchild_thresh>
 Node* NodeBuilder<nchild_thresh>::constructNodePtr() {
   // maybeCheckType() can throw an exception. Make sure to call the destructor
   // on the exception branch.
-  PtrCloser<Node> np(new Node(constructNV()));
+  std::unique_ptr<Node> np(new Node(constructNV()));
   maybeCheckType(*np.get());
   return np.release();
 }
 
 template <unsigned nchild_thresh>
 Node* NodeBuilder<nchild_thresh>::constructNodePtr() const {
-  PtrCloser<Node> np(new Node(constructNV()));
+  std::unique_ptr<Node> np(new Node(constructNV()));
   maybeCheckType(*np.get());
   return np.release();
 }
@@ -1313,7 +1303,8 @@ void NodeBuilder<nchild_thresh>::internalCopy(const NodeBuilder<N>& nb) {
   }
 
   Assert(nb.d_nvMaxChildren <= d_nvMaxChildren);
-  Assert(nb.d_nv->nv_end() - nb.d_nv->nv_begin() <= d_nvMaxChildren, "realloced:%s, d_nvMax:%u, size:%u, nc:%u", realloced ? "true" : "false", d_nvMaxChildren, nb.d_nv->nv_end() - nb.d_nv->nv_begin(), nb.d_nv->getNumChildren());
+  Assert(nb.d_nv->nv_end() >= nb.d_nv->nv_begin());
+  Assert((size_t)(nb.d_nv->nv_end() - nb.d_nv->nv_begin()) <= d_nvMaxChildren, "realloced:%s, d_nvMax:%u, size:%u, nc:%u", realloced ? "true" : "false", d_nvMaxChildren, nb.d_nv->nv_end() - nb.d_nv->nv_begin(), nb.d_nv->getNumChildren());
   std::copy(nb.d_nv->nv_begin(),
             nb.d_nv->nv_end(),
             d_nv->nv_begin());
@@ -1330,7 +1321,7 @@ void NodeBuilder<nchild_thresh>::internalCopy(const NodeBuilder<N>& nb) {
 #ifdef CVC4_DEBUG
 template <unsigned nchild_thresh>
 inline void NodeBuilder<nchild_thresh>::maybeCheckType(const TNode n) const
-    throw(TypeCheckingExceptionPrivate, AssertionException) {
+{
   /* force an immediate type check, if early type checking is
      enabled and the current node isn't a variable or constant */
   if( d_nm->getOptions()[options::earlyTypeChecking] ) {
