@@ -2,9 +2,9 @@
 /*! \file type.cpp
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Dejan Jovanovic, Andrew Reynolds
+ **   Morgan Deters, Dejan Jovanovic, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -329,20 +329,6 @@ bool Type::isSortConstructor() const {
   return d_typeNode->isSortConstructor();
 }
 
-/** Is this a predicate subtype */
-/* - not in release 1.0
-bool Type::isPredicateSubtype() const {
-  NodeManagerScope nms(d_nodeManager);
-  return d_typeNode->isPredicateSubtype();
-}
-*/
-
-/** Is this an integer subrange */
-bool Type::isSubrange() const {
-  NodeManagerScope nms(d_nodeManager);
-  return d_typeNode->isSubrange();
-}
-
 size_t FunctionType::getArity() const {
   return d_typeNode->getNumChildren() - 1;
 }
@@ -414,111 +400,96 @@ SortType SortConstructorType::instantiate(const std::vector<Type>& params) const
   return SortType(makeType(d_nodeManager->mkSort(*d_typeNode, paramsNodes)));
 }
 
-BooleanType::BooleanType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+BooleanType::BooleanType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isBoolean(), this);
 }
 
-IntegerType::IntegerType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+IntegerType::IntegerType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isInteger(), this);
 }
 
-RealType::RealType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+RealType::RealType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isReal(), this);
 }
 
-StringType::StringType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+StringType::StringType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isString(), this);
 }
 
-RegExpType::RegExpType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+RegExpType::RegExpType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isRegExp(), this);
 }
 
-RoundingModeType::RoundingModeType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+RoundingModeType::RoundingModeType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isRoundingMode(), this);
 }
 
-BitVectorType::BitVectorType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+BitVectorType::BitVectorType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isBitVector(), this);
 }
 
-FloatingPointType::FloatingPointType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+FloatingPointType::FloatingPointType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isFloatingPoint(), this);
 }
 
-DatatypeType::DatatypeType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+DatatypeType::DatatypeType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isDatatype(), this);
 }
 
-ConstructorType::ConstructorType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+ConstructorType::ConstructorType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isConstructor(), this);
 }
 
-SelectorType::SelectorType(const Type& t) throw(IllegalArgumentException) :
-  Type(t) {
+SelectorType::SelectorType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isSelector(), this);
 }
 
-TesterType::TesterType(const Type& t) throw(IllegalArgumentException)
-    : Type(t) {
+TesterType::TesterType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isTester(), this);
 }
 
-FunctionType::FunctionType(const Type& t) throw(IllegalArgumentException)
-    : Type(t) {
+FunctionType::FunctionType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isFunction(), this);
 }
 
-SExprType::SExprType(const Type& t) throw(IllegalArgumentException)
-    : Type(t) {
+SExprType::SExprType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isSExpr(), this);
 }
 
-ArrayType::ArrayType(const Type& t) throw(IllegalArgumentException)
-    : Type(t) {
+ArrayType::ArrayType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isArray(), this);
 }
 
-SetType::SetType(const Type& t) throw(IllegalArgumentException)
-    : Type(t) {
+SetType::SetType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isSet(), this);
 }
 
-SortType::SortType(const Type& t) throw(IllegalArgumentException)
-    : Type(t) {
+SortType::SortType(const Type& t) : Type(t)
+{
   PrettyCheckArgument(isNull() || isSort(), this);
 }
 
 SortConstructorType::SortConstructorType(const Type& t)
-  throw(IllegalArgumentException)
     : Type(t) {
   PrettyCheckArgument(isNull() || isSortConstructor(), this);
 }
 
-/* - not in release 1.0
-PredicateSubtype::PredicateSubtype(const Type& t)
-  throw(IllegalArgumentException) :
-  Type(t) {
-  PrettyCheckArgument(isNull() || isPredicateSubtype(), this);
-}
-*/
-
-SubrangeType::SubrangeType(const Type& t)
-  throw(IllegalArgumentException) :
-  Type(t) {
-  PrettyCheckArgument(isNull() || isSubrange(), this);
-}
-
 unsigned BitVectorType::getSize() const {
   return d_typeNode->getBitVectorSize();
 }
@@ -666,11 +637,6 @@ Type PredicateSubtype::getParentType() const {
 }
 */
 
-SubrangeBounds SubrangeType::getSubrangeBounds() const {
-  NodeManagerScope nms(d_nodeManager);
-  return d_typeNode->getSubrangeBounds();
-}
-
 size_t TypeHashFunction::operator()(const Type& t) const {
   return TypeNodeHashFunction()(NodeManager::fromType(t));
 }
