@@ -2,9 +2,9 @@
 /*! \file type_checker.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Paul Meng, Tim King
+ **   Morgan Deters, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -27,15 +27,13 @@ namespace expr {
 
 class TypeChecker {
 public:
+ static TypeNode computeType(NodeManager* nodeManager,
+                             TNode n,
+                             bool check = false);
 
-  static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check = false)
-    throw (TypeCheckingExceptionPrivate, AssertionException);
+ static bool computeIsConst(NodeManager* nodeManager, TNode n);
 
-  static bool computeIsConst(NodeManager* nodeManager, TNode n)
-    throw (AssertionException);
-
-  static bool neverIsConst(NodeManager* nodeManager, TNode n)
-    throw (AssertionException);
+ static bool neverIsConst(NodeManager* nodeManager, TNode n);
 
 };/* class TypeChecker */
 
