@@ -1,17 +1,18 @@
 /*********************                                                        */
-/*! \file base_options_template.h
+/*! \file module_template.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Paul Meng, Tim King
+ **   Mathias Preiner
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
  **
  ** \brief Contains code for handling command-line options.
  **
- ** Contains code for handling command-line options
+ ** For each <module>_options.toml configuration file, mkoptions.py
+ ** expands this template and generates a <module>_options.h file.
  **/
 
 #include "cvc4_private.h"
@@ -22,11 +23,10 @@
 #include "options/options.h"
 
 
-#line 26 "../../../src/options/base_options_template.h"
+
 
 #define CVC4_OPTIONS__FP__FOR_OPTION_HOLDER
 
-#line 30 "../../../src/options/base_options_template.h"
 
 namespace CVC4 {
 
@@ -34,22 +34,18 @@ namespace options {
 
 
 
-#line 38 "../../../src/options/base_options_template.h"
 
-}/* CVC4::options namespace */
+}  // namespace options
 
 
 
-#line 44 "../../../src/options/base_options_template.h"
 
 namespace options {
 
 
 
-#line 50 "../../../src/options/base_options_template.h"
 
-}/* CVC4::options namespace */
-
-}/* CVC4 namespace */
+}  // namespace options
+}  // namespace CVC4
 
 #endif /* __CVC4__OPTIONS__FP_H */
