@@ -2,9 +2,9 @@
 /*! \file option_exception.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Paul Meng, Tim King
+ **   Morgan Deters, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -30,9 +30,10 @@ namespace CVC4 {
  * class, below) should be used instead.
  */
 class CVC4_PUBLIC OptionException : public CVC4::Exception {
-public:
-  OptionException(const std::string& s) throw() :
-    CVC4::Exception("Error in option parsing: " + s) {
+ public:
+  OptionException(const std::string& s)
+      : CVC4::Exception("Error in option parsing: " + s)
+  {
   }
 };/* class OptionException */
 
@@ -41,7 +42,7 @@ public:
  * unrecognized or unsupported option key.
  */
 class CVC4_PUBLIC UnrecognizedOptionException : public CVC4::OptionException {
-public:
+ public:
   UnrecognizedOptionException() :
     CVC4::OptionException("Unrecognized informational or option key or setting") {
   }
