@@ -4,7 +4,7 @@
  ** Top contributors (to current version):
  **   Christopher L. Conway, Morgan Deters, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -65,7 +65,7 @@ void ParserBuilder::init(ExprManager* exprManager,
 }
 
 Parser* ParserBuilder::build()
-  throw (InputStreamException) {
+{
   Input* input = NULL;
   switch( d_inputType ) {
   case FILE_INPUT:
@@ -91,11 +91,6 @@ Parser* ParserBuilder::build()
   case language::input::LANG_SMTLIB_V1:
     parser = new Smt1(d_exprManager, input, d_strictMode, d_parseOnly);
     break;
-  case language::input::LANG_SMTLIB_V2_0:
-  case language::input::LANG_SMTLIB_V2_5:
-  case language::input::LANG_SMTLIB_V2_6:
-    parser = new Smt2(d_exprManager, input, d_strictMode, d_parseOnly);
-    break;
   case language::input::LANG_SYGUS:
     parser = new Smt2(d_exprManager, input, d_strictMode, d_parseOnly);
     break;
@@ -103,7 +98,14 @@ Parser* ParserBuilder::build()
     parser = new Tptp(d_exprManager, input, d_strictMode, d_parseOnly);
     break;
   default:
-    parser = new Parser(d_exprManager, input, d_strictMode, d_parseOnly);
+    if (language::isInputLang_smt2(d_lang))
+    {
+      parser = new Smt2(d_exprManager, input, d_strictMode, d_parseOnly);
+    }
+    else
+    {
+      parser = new Parser(d_exprManager, input, d_strictMode, d_parseOnly);
+    }
     break;
   }
 
