@@ -2,9 +2,9 @@
 /*! \file bvminisat.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Dejan Jovanovic, Liana Hadarean, Tim King
+ **   Dejan Jovanovic, Mathias Preiner, Liana Hadarean
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -18,6 +18,8 @@
 
 #pragma once
 
+#include <memory>
+
 #include "context/cdo.h"
 #include "proof/clause_id.h"
 #include "prop/bvminisat/simp/SimpSolver.h"
@@ -27,82 +29,76 @@
 namespace CVC4 {
 namespace prop {
 
-class BVMinisatSatSolver : public BVSatSolverInterface, public context::ContextNotifyObj {
-
-private:
-
-  class MinisatNotify : public BVMinisat::Notify {
+class BVMinisatSatSolver : public BVSatSolverInterface,
+                           public context::ContextNotifyObj
+{
+ private:
+  class MinisatNotify : public BVMinisat::Notify
+  {
     BVSatSolverInterface::Notify* d_notify;
-  public:
-    MinisatNotify(BVSatSolverInterface::Notify* notify)
-    : d_notify(notify)
-    {}
-    bool notify(BVMinisat::Lit lit) {
+
+   public:
+    MinisatNotify(BVSatSolverInterface::Notify* notify) : d_notify(notify) {}
+    bool notify(BVMinisat::Lit lit) override
+    {
       return d_notify->notify(toSatLiteral(lit));
     }
-    void notify(BVMinisat::vec<BVMinisat::Lit>& clause) {
-      SatClause satClause;
-      for (int i = 0; i < clause.size(); ++i) {
-        satClause.push_back(toSatLiteral(clause[i])); 
-      }
-      d_notify->notify(satClause);
-    }
-
-    void spendResource(unsigned ammount) {
-      d_notify->spendResource(ammount);
-    }
-    void safePoint(unsigned ammount) {
-      d_notify->safePoint(ammount);
+    void notify(BVMinisat::vec<BVMinisat::Lit>& clause) override;
+    void spendResource(unsigned amount) override
+    {
+      d_notify->spendResource(amount);
     }
+    void safePoint(unsigned amount) override { d_notify->safePoint(amount); }
   };
 
-  BVMinisat::SimpSolver* d_minisat;
-  MinisatNotify* d_minisatNotify;
+	std::unique_ptr<BVMinisat::SimpSolver> d_minisat;
+	std::unique_ptr<MinisatNotify> d_minisatNotify;
 
   unsigned d_assertionsCount;
   context::CDO<unsigned> d_assertionsRealCount;
   context::CDO<unsigned> d_lastPropagation;
 
 protected:
-
-  void contextNotifyPop();
+ void contextNotifyPop() override;
 
 public:
 
   BVMinisatSatSolver(StatisticsRegistry* registry, context::Context* mainSatContext, const std::string& name = "");
   virtual ~BVMinisatSatSolver();
 
-  void setNotify(Notify* notify);
+  void setNotify(Notify* notify) override;
 
-  ClauseId addClause(SatClause& clause, bool removable);
+  ClauseId addClause(SatClause& clause, bool removable) override;
 
-  ClauseId addXorClause(SatClause& clause, bool rhs, bool removable) {
+  ClauseId addXorClause(SatClause& clause, bool rhs, bool removable) override
+  {
     Unreachable("Minisat does not support native XOR reasoning");
   }
-  
-  SatValue propagate();
 
-  SatVariable newVar(bool isTheoryAtom = false, bool preRegister = false, bool canErase = true);
+  SatValue propagate() override;
 
-  SatVariable trueVar() { return d_minisat->trueVar(); }
-  SatVariable falseVar() { return d_minisat->falseVar(); }
+  SatVariable newVar(bool isTheoryAtom = false,
+                     bool preRegister = false,
+                     bool canErase = true) override;
 
-  void markUnremovable(SatLiteral lit);
+  SatVariable trueVar() override { return d_minisat->trueVar(); }
+  SatVariable falseVar() override { return d_minisat->falseVar(); }
 
-  void interrupt();
+  void markUnremovable(SatLiteral lit) override;
 
-  SatValue solve();
-  SatValue solve(long unsigned int&);
-  bool ok() const; 
-  void getUnsatCore(SatClause& unsatCore);
+  void interrupt() override;
 
-  SatValue value(SatLiteral l);
-  SatValue modelValue(SatLiteral l);
+  SatValue solve() override;
+  SatValue solve(long unsigned int&) override;
+  bool ok() const override;
+  void getUnsatCore(SatClause& unsatCore) override;
+
+  SatValue value(SatLiteral l) override;
+  SatValue modelValue(SatLiteral l) override;
 
   void unregisterVar(SatLiteral lit);
   void renewVar(SatLiteral lit, int level = -1);
-  unsigned getAssertionLevel() const;
-
+  unsigned getAssertionLevel() const override;
 
   // helper methods for converting from the internal Minisat representation
 
@@ -113,17 +109,17 @@ public:
 
   static void  toMinisatClause(SatClause& clause, BVMinisat::vec<BVMinisat::Lit>& minisat_clause);
   static void  toSatClause    (const BVMinisat::Clause& clause, SatClause& sat_clause);
-  void addMarkerLiteral(SatLiteral lit);
+  void addMarkerLiteral(SatLiteral lit) override;
 
-  void explain(SatLiteral lit, std::vector<SatLiteral>& explanation);
+  void explain(SatLiteral lit, std::vector<SatLiteral>& explanation) override;
 
-  SatValue assertAssumption(SatLiteral lit, bool propagate);
+  SatValue assertAssumption(SatLiteral lit, bool propagate) override;
 
-  void popAssumption();
-  
-  void setProofLog( BitVectorProof * bvp );
+  void popAssumption() override;
 
-private:
+  void setProofLog(BitVectorProof* bvp) override;
+
+ private:
   /* Disable the default constructor. */
   BVMinisatSatSolver() CVC4_UNDEFINED;
 
@@ -137,7 +133,7 @@ private:
     ReferenceStat<uint64_t> d_statTotLiterals;
     ReferenceStat<int> d_statEliminatedVars;
     IntStat d_statCallsToSolve;
-    BackedStat<double> d_statSolveTime;
+    TimerStat d_statSolveTime;
     bool d_registerStats;
     Statistics(StatisticsRegistry* registry, const std::string& prefix);
     ~Statistics();
