@@ -21,7 +21,6 @@ OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWA
 #ifndef BVMinisat_Solver_h
 #define BVMinisat_Solver_h
 
-#include <ext/hash_set>
 #include <vector>
 
 #include "context/context.h"
@@ -62,8 +61,8 @@ public:
    */
   virtual void notify(vec<Lit>& learnt) = 0;
 
-  virtual void spendResource(unsigned ammount) = 0;
-  virtual void safePoint(unsigned ammount) = 0;
+  virtual void spendResource(unsigned amount) = 0;
+  virtual void safePoint(unsigned amount) = 0;
 };
 
 //=================================================================================================
@@ -81,7 +80,7 @@ public:
     static CRef TCRef_Lazy;
 private:
     /** To notify */
-    Notify* notify;
+    Notify* d_notify;
 
     /** Cvc4 context */
     CVC4::context::Context* c;
@@ -99,7 +98,7 @@ public:
     Solver(CVC4::context::Context* c);
     virtual ~Solver();
 
-    void setNotify(Notify* toNotify) { notify = toNotify; }
+    void setNotify(Notify* toNotify);
 
     // Problem specification:
     //
@@ -345,7 +344,7 @@ protected:
     CRef     reason           (Var x) const;
     int      level            (Var x) const;
     double   progressEstimate ()      const; // DELETE THIS ?? IT'S NOT VERY USEFUL ...
-    bool     withinBudget     (uint64_t ammount)      const;
+    bool     withinBudget     (uint64_t amount)      const;
 
     // Static helpers:
     //
@@ -462,14 +461,6 @@ inline void     Solver::setPropBudget(int64_t x){ propagation_budget = propagati
 inline void     Solver::interrupt(){ asynch_interrupt = true; }
 inline void     Solver::clearInterrupt(){ asynch_interrupt = false; }
 inline void     Solver::budgetOff(){ conflict_budget = propagation_budget = -1; }
-inline bool     Solver::withinBudget(uint64_t ammount) const {
-    Assert (notify);
-    notify->spendResource(ammount);
-    notify->safePoint(0);
-
-    return !asynch_interrupt &&
-           (conflict_budget    < 0 || conflicts < (uint64_t)conflict_budget) &&
-           (propagation_budget < 0 || propagations < (uint64_t)propagation_budget); }
 
 inline lbool     Solver::solve         ()                    { budgetOff(); return solve_(); }
 inline lbool     Solver::solve         (Lit p)               { budgetOff(); assumptions.push(p); return solve_(); }
