@@ -69,8 +69,7 @@ class SimpSolver : public Solver {
 
     // Memory managment:
     //
-    virtual void garbageCollect();
-
+    void garbageCollect() override;
 
     // Generate a (possibly simplified) DIMACS file:
     //
