@@ -2,9 +2,9 @@
 /*! \file cnf_stream.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Tim King, Dejan Jovanovic, Morgan Deters
+ **   Dejan Jovanovic, Tim King, Morgan Deters
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -25,8 +25,6 @@
 #ifndef __CVC4__PROP__CNF_STREAM_H
 #define __CVC4__PROP__CNF_STREAM_H
 
-#include <ext/hash_map>
-
 #include "context/cdinsert_hashmap.h"
 #include "context/cdlist.h"
 #include "expr/node.h"
@@ -288,8 +286,11 @@ class TseitinCnfStream : public CnfStream {
    * @param removable is this something that can be erased
    * @param negated true if negated
    */
-  void convertAndAssert(TNode node, bool removable, bool negated,
-                        ProofRule rule, TNode from = TNode::null());
+  void convertAndAssert(TNode node,
+                        bool removable,
+                        bool negated,
+                        ProofRule rule,
+                        TNode from = TNode::null()) override;
 
  private:
   /**
@@ -330,7 +331,7 @@ class TseitinCnfStream : public CnfStream {
    */
   SatLiteral toCNF(TNode node, bool negated = false);
 
-  void ensureLiteral(TNode n, bool noPreregistration = false);
+  void ensureLiteral(TNode n, bool noPreregistration = false) override;
 
 }; /* class TseitinCnfStream */
 
