@@ -2,9 +2,9 @@
 /*! \file approx_simplex.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Tim King, Morgan Deters, Paul Meng
+ **   Tim King, Morgan Deters
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -24,6 +24,7 @@
 #include "theory/arith/arithvar.h"
 #include "theory/arith/delta_rational.h"
 #include "util/dense_map.h"
+#include "util/maybe.h"
 #include "util/rational.h"
 #include "util/statistics_registry.h"
 
@@ -49,26 +50,9 @@ enum MipResult {
 std::ostream& operator<<(std::ostream& out, MipResult res);
 
 class ApproximateStatistics {
-public:
-  // IntStat d_relaxCalls;
-  // IntStat d_relaxUnknowns;
-  // IntStat d_relaxFeasible;
-  // IntStat d_relaxInfeasible;
-  // IntStat d_relaxPivotsExhausted;
-
-  // IntStat d_mipCalls;
-  // IntStat d_mipUnknowns;
-  // IntStat d_mipBingo;
-  // IntStat d_mipClosed;
-  // IntStat d_mipBranchesExhausted;
-  // IntStat d_mipPivotsExhausted;
-  // IntStat d_mipExecExhausted;
-
-
-  // IntStat d_gmiGen;
-  // IntStat d_gmiReplay;
-  // IntStat d_mipGen;
-  // IntStat d_mipReplay;
+ public:
+  ApproximateStatistics();
+  ~ApproximateStatistics();
 
   IntStat d_branchMaxDepth;
   IntStat d_branchesMaxOnAVar;
@@ -76,9 +60,6 @@ public:
   TimerStat d_gaussianElimConstructTime;
   IntStat d_gaussianElimConstruct;
   AverageStat d_averageGuesses;
-
-  ApproximateStatistics();
-  ~ApproximateStatistics();
 };
 
 
@@ -89,26 +70,7 @@ class CutInfo;
 class RowsDeleted;
 
 class ApproximateSimplex{
-protected:
-  const ArithVariables& d_vars;
-  TreeLog& d_log;
-  ApproximateStatistics& d_stats;
-
-  int d_pivotLimit;
-  /* the maximum pivots allowed in a query. */
-
-  int d_branchLimit;
-  /* maximum branches allowed on a variable */
-
-  int d_maxDepth;
-  /* maxmimum branching depth allowed.*/
-
-  static Integer s_defaultMaxDenom;
-  /* Default denominator for diophatine approximation.
-  * 2^{26}*/
-
-public:
-
+ public:
   static bool enabled();
 
   /**
@@ -129,7 +91,6 @@ public:
   void setBranchingDepth(int bd);
 
   /** A result is either sat, unsat or unknown.*/
-  //enum ApproxResult {ApproxError, ApproxSat, ApproxUnsat};
   struct Solution {
     DenseSet newBasis;
     DenseMap<DeltaRational> newValues;
@@ -137,8 +98,6 @@ public:
   };
 
   virtual ArithVar getBranchVar(const NodeLog& nl) const = 0;
-  //virtual void mapRowId(int nid, int ind, ArithVar v) = 0;
-  //virtual void applyRowsDeleted(int nid, const RowsDeleted& rd) = 0;
 
   /** Sets a maximization criteria for the approximate solver.*/
   virtual void setOptCoeffs(const ArithRatPairVec& ref) = 0;
@@ -146,17 +105,15 @@ public:
   virtual ArithRatPairVec heuristicOptCoeffs() const = 0;
 
   virtual LinResult solveRelaxation() = 0;
-  virtual Solution extractRelaxation() const throw(RationalFromDoubleException) = 0;
+  virtual Solution extractRelaxation() const = 0;
 
   virtual MipResult solveMIP(bool activelyLog) = 0;
-  virtual Solution extractMIP() const throw(RationalFromDoubleException) = 0;
 
-  virtual std::vector<const CutInfo*> getValidCuts(const NodeLog& node) throw(RationalFromDoubleException) = 0;
-  //virtual std::vector<const NodeLog*> getBranches() = 0;
+  virtual Solution extractMIP() const = 0;
 
-  //virtual Node downBranchLiteral(const NodeLog& con) const = 0;
+  virtual std::vector<const CutInfo*> getValidCuts(const NodeLog& node) = 0;
 
-  virtual void tryCut(int nid, CutInfo& cut) throw(RationalFromDoubleException) = 0;
+  virtual void tryCut(int nid, CutInfo& cut) = 0;
 
   /** UTILITIES FOR DEALING WITH ESTIMATES */
 
@@ -171,8 +128,8 @@ public:
    * cuts off the estimate once the value is approximately zero.
    * This is designed for removing rounding artifacts.
    */
-  static Rational estimateWithCFE(double d) throw(RationalFromDoubleException);
-  static Rational estimateWithCFE(double d, const Integer& D) throw(RationalFromDoubleException);
+  static Maybe<Rational> estimateWithCFE(double d);
+  static Maybe<Rational> estimateWithCFE(double d, const Integer& D);
 
   /**
    * Converts a rational to a continued fraction expansion representation
@@ -185,10 +142,26 @@ public:
   static Rational cfeToRational(const std::vector<Integer>& exp);
 
   /** Estimates a rational as a continued fraction expansion.*/
-  //static Rational estimateWithCFE(const Rational& q, int depth);
   static Rational estimateWithCFE(const Rational& q, const Integer& K);
 
   virtual double sumInfeasibilities(bool mip) const = 0;
+
+ protected:
+  const ArithVariables& d_vars;
+  TreeLog& d_log;
+  ApproximateStatistics& d_stats;
+
+  /* the maximum pivots allowed in a query. */
+  int d_pivotLimit;
+
+  /* maximum branches allowed on a variable */
+  int d_branchLimit;
+
+  /* maxmimum branching depth allowed.*/
+  int d_maxDepth;
+
+  /* Default denominator for diophatine approximation, 2^{26} .*/
+  static Integer s_defaultMaxDenom;
 };/* class ApproximateSimplex */
 
 
