@@ -2,9 +2,9 @@
 /*! \file matrix.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Tim King, Paul Meng, Morgan Deters
+ **   Tim King, Mathias Preiner, Morgan Deters
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -48,9 +48,9 @@ public:
 
 class NoEffectCCCB : public CoefficientChangeCallback {
 public:
-  void update(RowIndex ridx, ArithVar nb, int oldSgn, int currSgn);
-  void multiplyRow(RowIndex ridx, int Sgn);
-  bool canUseRow(RowIndex ridx) const;
+ void update(RowIndex ridx, ArithVar nb, int oldSgn, int currSgn) override;
+ void multiplyRow(RowIndex ridx, int Sgn) override;
+ bool canUseRow(RowIndex ridx) const override;
 };
 
 template<class T>
