@@ -2,9 +2,9 @@
 /*! \file theory_idl.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Dejan Jovanovic, Morgan Deters, Paul Meng
+ **   Dejan Jovanovic, Mathias Preiner, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -48,13 +48,13 @@ public:
             Valuation valuation, const LogicInfo& logicInfo);
 
   /** Pre-processing of input atoms */
-  Node ppRewrite(TNode atom);
+  Node ppRewrite(TNode atom) override;
 
   /** Check the assertions for satisfiability */
-  void check(Effort effort);
+  void check(Effort effort) override;
 
   /** Identity string */
-  std::string identify() const { return "THEORY_IDL"; }
+  std::string identify() const override { return "THEORY_IDL"; }
 
 };/* class TheoryIdl */
 
