@@ -4,7 +4,7 @@
  ** Top contributors (to current version):
  **   Morgan Deters, Tim King, Andrew Reynolds
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -30,48 +30,54 @@ using namespace CVC4::theory;
 
 namespace CVC4 {
 
-LogicInfo::LogicInfo() :
-  d_logicString(""),
-  d_theories(THEORY_LAST, false),
-  d_sharingTheories(0),
-  d_integers(true),
-  d_reals(true),
-  d_linear(false),
-  d_differenceLogic(false),
-  d_cardinalityConstraints(false),
-  d_locked(false) {
-
+LogicInfo::LogicInfo()
+    : d_logicString(""),
+      d_theories(THEORY_LAST, false),
+      d_sharingTheories(0),
+      d_integers(true),
+      d_reals(true),
+      d_transcendentals(true),
+      d_linear(false),
+      d_differenceLogic(false),
+      d_cardinalityConstraints(false),
+      d_higherOrder(true),
+      d_locked(false)
+{
   for(TheoryId id = THEORY_FIRST; id < THEORY_LAST; ++id) {
     enableTheory(id);
   }
 }
 
-LogicInfo::LogicInfo(std::string logicString) throw(IllegalArgumentException) :
-  d_logicString(""),
-  d_theories(THEORY_LAST, false),
-  d_sharingTheories(0),
-  d_integers(false),
-  d_reals(false),
-  d_linear(false),
-  d_differenceLogic(false),
-  d_cardinalityConstraints(false),
-  d_locked(false) {
-
+LogicInfo::LogicInfo(std::string logicString)
+    : d_logicString(""),
+      d_theories(THEORY_LAST, false),
+      d_sharingTheories(0),
+      d_integers(false),
+      d_reals(false),
+      d_transcendentals(false),
+      d_linear(false),
+      d_differenceLogic(false),
+      d_cardinalityConstraints(false),
+      d_higherOrder(false),
+      d_locked(false)
+{
   setLogicString(logicString);
   lock();
 }
 
-LogicInfo::LogicInfo(const char* logicString) throw(IllegalArgumentException) :
-  d_logicString(""),
-  d_theories(THEORY_LAST, false),
-  d_sharingTheories(0),
-  d_integers(false),
-  d_reals(false),
-  d_linear(false),
-  d_differenceLogic(false),
-  d_cardinalityConstraints(false),
-  d_locked(false) {
-
+LogicInfo::LogicInfo(const char* logicString)
+    : d_logicString(""),
+      d_theories(THEORY_LAST, false),
+      d_sharingTheories(0),
+      d_integers(false),
+      d_reals(false),
+      d_transcendentals(false),
+      d_linear(false),
+      d_differenceLogic(false),
+      d_cardinalityConstraints(false),
+      d_higherOrder(false),
+      d_locked(false)
+{
   setLogicString(logicString);
   lock();
 }
@@ -98,6 +104,15 @@ bool LogicInfo::isQuantified() const {
   return isTheoryEnabled(theory::THEORY_QUANTIFIERS);
 }
 
+/** Is this a higher-order logic? */
+bool LogicInfo::isHigherOrder() const
+{
+  PrettyCheckArgument(d_locked,
+                      *this,
+                      "This LogicInfo isn't locked yet, and cannot be queried");
+  return d_higherOrder;
+}
+
 /** Is this the all-inclusive logic? */
 bool LogicInfo::hasEverything() const {
   PrettyCheckArgument(d_locked, *this,
@@ -144,6 +159,18 @@ bool LogicInfo::areRealsUsed() const {
   return d_reals;
 }
 
+bool LogicInfo::areTranscendentalsUsed() const
+{
+  PrettyCheckArgument(d_locked,
+                      *this,
+                      "This LogicInfo isn't locked yet, and cannot be queried");
+  PrettyCheckArgument(isTheoryEnabled(theory::THEORY_ARITH),
+                      *this,
+                      "Arithmetic not used in this LogicInfo; cannot ask "
+                      "whether transcendentals are used");
+  return d_transcendentals;
+}
+
 bool LogicInfo::isLinear() const {
   PrettyCheckArgument(d_locked, *this,
                       "This LogicInfo isn't locked yet, and cannot be queried");
@@ -181,11 +208,10 @@ bool LogicInfo::operator==(const LogicInfo& other) const {
   PrettyCheckArgument(d_sharingTheories == other.d_sharingTheories, *this,
                       "LogicInfo internal inconsistency");
   if(isTheoryEnabled(theory::THEORY_ARITH)) {
-    return
-        d_integers == other.d_integers &&
-        d_reals == other.d_reals &&
-        d_linear == other.d_linear &&
-        d_differenceLogic == other.d_differenceLogic;
+    return d_integers == other.d_integers && d_reals == other.d_reals
+           && d_transcendentals == other.d_transcendentals
+           && d_linear == other.d_linear
+           && d_differenceLogic == other.d_differenceLogic;
   } else {
     return true;
   }
@@ -202,11 +228,10 @@ bool LogicInfo::operator<=(const LogicInfo& other) const {
   PrettyCheckArgument(d_sharingTheories <= other.d_sharingTheories, *this,
                       "LogicInfo internal inconsistency");
   if(isTheoryEnabled(theory::THEORY_ARITH) && other.isTheoryEnabled(theory::THEORY_ARITH)) {
-    return
-        (!d_integers || other.d_integers) &&
-        (!d_reals || other.d_reals) &&
-        (d_linear || !other.d_linear) &&
-        (d_differenceLogic || !other.d_differenceLogic);
+    return (!d_integers || other.d_integers) && (!d_reals || other.d_reals)
+           && (!d_transcendentals || other.d_transcendentals)
+           && (d_linear || !other.d_linear)
+           && (d_differenceLogic || !other.d_differenceLogic);
   } else {
     return true;
   }
@@ -223,11 +248,10 @@ bool LogicInfo::operator>=(const LogicInfo& other) const {
   PrettyCheckArgument(d_sharingTheories >= other.d_sharingTheories, *this,
                       "LogicInfo internal inconsistency");
   if(isTheoryEnabled(theory::THEORY_ARITH) && other.isTheoryEnabled(theory::THEORY_ARITH)) {
-    return
-        (d_integers || !other.d_integers) &&
-        (d_reals || !other.d_reals) &&
-        (!d_linear || other.d_linear) &&
-        (!d_differenceLogic || other.d_differenceLogic);
+    return (d_integers || !other.d_integers) && (d_reals || !other.d_reals)
+           && (d_transcendentals || !other.d_transcendentals)
+           && (!d_linear || other.d_linear)
+           && (!d_differenceLogic || other.d_differenceLogic);
     } else {
     return true;
   }
@@ -252,7 +276,7 @@ std::string LogicInfo::getLogicString() const {
       if(!isQuantified()) {
         ss << "QF_";
       }
-      if(d_theories[THEORY_ARRAY]) {
+      if(d_theories[THEORY_ARRAYS]) {
         ss << (d_sharingTheories == 1 ? "AX" : "A");
         ++seen;
       }
@@ -289,6 +313,7 @@ std::string LogicInfo::getLogicString() const {
           ss << (areIntegersUsed() ? "I" : "");
           ss << (areRealsUsed() ? "R" : "");
           ss << "A";
+          ss << (areTranscendentalsUsed() ? "T" : "");
         }
         ++seen;
       }
@@ -315,7 +340,8 @@ std::string LogicInfo::getLogicString() const {
   return d_logicString;
 }
 
-void LogicInfo::setLogicString(std::string logicString) throw(IllegalArgumentException) {
+void LogicInfo::setLogicString(std::string logicString)
+{
   PrettyCheckArgument(!d_locked, *this,
                       "This LogicInfo is locked, and cannot be modified");
   for(TheoryId id = THEORY_FIRST; id < THEORY_LAST; ++id) {
@@ -336,6 +362,10 @@ void LogicInfo::setLogicString(std::string logicString) throw(IllegalArgumentExc
   } else if(!strcmp(p, "QF_SAT")) {
     // propositional logic only; we're done.
     p += 6;
+  } else if(!strcmp(p, "SAT")) {
+    // quantified Boolean formulas only; we're done.
+    enableQuantifiers();
+    p += 3;
   } else if(!strcmp(p, "QF_ALL_SUPPORTED")) {
     // the "all theories included" logic, no quantifiers
     enableEverything();
@@ -360,6 +390,14 @@ void LogicInfo::setLogicString(std::string logicString) throw(IllegalArgumentExc
     enableQuantifiers();
     arithNonLinear();
     p += 3;
+  }
+  else if (!strcmp(p, "HORN"))
+  {
+    // the HORN logic
+    enableEverything();
+    enableQuantifiers();
+    arithNonLinear();
+    p += 4;
   } else {
     if(!strncmp(p, "QF_", 3)) {
       disableQuantifiers();
@@ -368,11 +406,11 @@ void LogicInfo::setLogicString(std::string logicString) throw(IllegalArgumentExc
       enableQuantifiers();
     }
     if(!strncmp(p, "AX", 2)) {
-      enableTheory(THEORY_ARRAY);
+      enableTheory(THEORY_ARRAYS);
       p += 2;
     } else {
       if(*p == 'A') {
-        enableTheory(THEORY_ARRAY);
+        enableTheory(THEORY_ARRAYS);
         ++p;
       }
       if(!strncmp(p, "UF", 2)) {
@@ -446,11 +484,21 @@ void LogicInfo::setLogicString(std::string logicString) throw(IllegalArgumentExc
         enableReals();
         arithNonLinear();
         p += 3;
+        if (*p == 'T')
+        {
+          arithTranscendentals();
+          p += 1;
+        }
       } else if(!strncmp(p, "NIRA", 4)) {
         enableIntegers();
         enableReals();
         arithNonLinear();
         p += 4;
+        if (*p == 'T')
+        {
+          arithTranscendentals();
+          p += 1;
+        }
       }
       if(!strncmp(p, "FS", 2)) {
         enableTheory(THEORY_SETS);
@@ -462,6 +510,16 @@ void LogicInfo::setLogicString(std::string logicString) throw(IllegalArgumentExc
       }
     }
   }
+
+  if (d_theories[THEORY_FP])
+  {
+    // THEORY_BV is needed for bit-blasting.
+    // We have to set this here rather than in expandDefinition as it
+    // is possible to create variables without any theory specific
+    // operations, so expandDefinition won't be called.
+    enableTheory(THEORY_BV);
+  }
+
   if(*p != '\0') {
     stringstream err;
     err << "LogicInfo::setLogicString(): ";
@@ -546,11 +604,28 @@ void LogicInfo::disableReals() {
   }
 }
 
+void LogicInfo::arithTranscendentals()
+{
+  PrettyCheckArgument(
+      !d_locked, *this, "This LogicInfo is locked, and cannot be modified");
+  d_logicString = "";
+  d_transcendentals = true;
+  if (!d_reals)
+  {
+    enableReals();
+  }
+  if (d_linear)
+  {
+    arithNonLinear();
+  }
+}
+
 void LogicInfo::arithOnlyDifference() {
   PrettyCheckArgument(!d_locked, *this, "This LogicInfo is locked, and cannot be modified");
   d_logicString = "";
   d_linear = true;
   d_differenceLogic = true;
+  d_transcendentals = false;
 }
 
 void LogicInfo::arithOnlyLinear() {
@@ -558,6 +633,7 @@ void LogicInfo::arithOnlyLinear() {
   d_logicString = "";
   d_linear = true;
   d_differenceLogic = false;
+  d_transcendentals = false;
 }
 
 void LogicInfo::arithNonLinear() {
@@ -567,6 +643,38 @@ void LogicInfo::arithNonLinear() {
   d_differenceLogic = false;
 }
 
+void LogicInfo::enableCardinalityConstraints()
+{
+  PrettyCheckArgument(
+      !d_locked, *this, "This LogicInfo is locked, and cannot be modified");
+  d_logicString = "";
+  d_cardinalityConstraints = true;
+}
+
+void LogicInfo::disableCardinalityConstraints()
+{
+  PrettyCheckArgument(
+      !d_locked, *this, "This LogicInfo is locked, and cannot be modified");
+  d_logicString = "";
+  d_cardinalityConstraints = false;
+}
+
+void LogicInfo::enableHigherOrder()
+{
+  PrettyCheckArgument(
+      !d_locked, *this, "This LogicInfo is locked, and cannot be modified");
+  d_logicString = "";
+  d_higherOrder = true;
+}
+
+void LogicInfo::disableHigherOrder()
+{
+  PrettyCheckArgument(
+      !d_locked, *this, "This LogicInfo is locked, and cannot be modified");
+  d_logicString = "";
+  d_higherOrder = false;
+}
+
 LogicInfo LogicInfo::getUnlockedCopy() const {
   if(d_locked) {
     LogicInfo info = *this;
