@@ -2,14 +2,14 @@
 /*! \file quant_conflict_find.cpp
  ** \verbatim
  ** Top contributors (to current version):
- **   Clark Barrett, Andrew Reynolds, Tim King
+ **   Andrew Reynolds, Tim King, Morgan Deters
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
  **
- ** \brief quant conflict find class
+ ** \brief Implements conflict-based instantiation (Reynolds et al FMCAD 2014)
  **
  **/
 
@@ -19,10 +19,12 @@
 
 #include "options/quantifiers_options.h"
 #include "smt/smt_statistics_registry.h"
+#include "theory/quantifiers/first_order_model.h"
+#include "theory/quantifiers/instantiate.h"
 #include "theory/quantifiers/quant_util.h"
 #include "theory/quantifiers/term_database.h"
-#include "theory/quantifiers/trigger.h"
-#include "theory/quantifiers/first_order_model.h"
+#include "theory/quantifiers/term_util.h"
+#include "theory/quantifiers/ematching/trigger.h"
 #include "theory/theory_engine.h"
 
 using namespace CVC4::kind;
@@ -32,9 +34,7 @@ namespace CVC4 {
 namespace theory {
 namespace quantifiers {
 
-QuantInfo::QuantInfo()
-    : d_mg( NULL )
-{}
+QuantInfo::QuantInfo() : d_unassigned_nvar(0), d_una_index(0), d_mg(nullptr) {}
 
 QuantInfo::~QuantInfo() {
   delete d_mg;
@@ -559,7 +559,7 @@ bool QuantInfo::isMatchSpurious( QuantConflictFind * p ) {
   for( int i=0; i<getNumVars(); i++ ){
     //std::map< int, TNode >::iterator it = d_match.find( i );
     if( !d_match[i].isNull() ){
-      if( !getCurrentCanBeEqual( p, i, d_match[i], p->d_effort==QuantConflictFind::effort_conflict ) ){
+      if (!getCurrentCanBeEqual(p, i, d_match[i], p->atConflictEffort())) {
         return true;
       }
     }
@@ -570,7 +570,7 @@ bool QuantInfo::isMatchSpurious( QuantConflictFind * p ) {
 bool QuantInfo::isTConstraintSpurious( QuantConflictFind * p, std::vector< Node >& terms ) {
   if( options::qcfEagerTest() ){
     //check whether the instantiation evaluates as expected
-    if( p->d_effort==QuantConflictFind::effort_conflict ){
+    if (p->atConflictEffort()) {
       Trace("qcf-instance-check") << "Possible conflict instance for " << d_q << " : " << std::endl;
       std::map< TNode, TNode > subs;
       for( unsigned i=0; i<terms.size(); i++ ){
@@ -587,7 +587,8 @@ bool QuantInfo::isTConstraintSpurious( QuantConflictFind * p, std::vector< Node
         return true;
       }
     }else{
-      Node inst = p->d_quantEngine->getInstantiation( d_q, terms );
+      Node inst =
+          p->d_quantEngine->getInstantiate()->getInstantiation(d_q, terms);
       Node inst_eval = p->getTermDatabase()->evaluateTerm( inst, NULL, options::qcfTConstraint() );
       if( Trace.isOn("qcf-instance-check") ){
         Trace("qcf-instance-check") << "Possible propagating instance for " << d_q << " : " << std::endl;
@@ -596,7 +597,7 @@ bool QuantInfo::isTConstraintSpurious( QuantConflictFind * p, std::vector< Node
         }
         Trace("qcf-instance-check") << "...evaluates to " << inst_eval << std::endl;
       }
-      if( inst_eval.isNull() || inst_eval==p->getTermDatabase()->d_true || !isPropagatingInstance( p, inst_eval ) ){
+      if( inst_eval.isNull() || inst_eval==p->getTermUtil()->d_true || !isPropagatingInstance( p, inst_eval ) ){
         Trace("qcf-instance-check") << "...spurious." << std::endl;
         return true;
       }else{
@@ -608,9 +609,10 @@ bool QuantInfo::isTConstraintSpurious( QuantConflictFind * p, std::vector< Node
     //check constraints
     for( std::map< Node, bool >::iterator it = d_tconstraints.begin(); it != d_tconstraints.end(); ++it ){
       //apply substitution to the tconstraint
-      Node cons = p->getTermDatabase()->getInstantiatedNode( it->first, d_q, terms );
+      Node cons =
+          p->getTermUtil()->substituteBoundVariables(it->first, d_q, terms);
       cons = it->second ? cons : cons.negate();
-      if( !entailmentTest( p, cons, p->d_effort==QuantConflictFind::effort_conflict ) ){
+      if (!entailmentTest(p, cons, p->atConflictEffort())) {
         return true;
       }
     }
@@ -719,7 +721,7 @@ bool QuantInfo::completeMatch( QuantConflictFind * p, std::vector< int >& assign
               if( slv_v==-1 ){
                 Trace("qcf-tconstraint-debug") << "...will solve for var #" << vn << std::endl;
                 slv_v = vn;
-                if( p->d_effort!=QuantConflictFind::effort_conflict ){
+                if (!p->atConflictEffort()) {
                   break;
                 }
               }else{
@@ -756,7 +758,7 @@ bool QuantInfo::completeMatch( QuantConflictFind * p, std::vector< int >& assign
             if( v==d_vars[index] ){
               sum = lhs;
             }else{
-              if( p->d_effort==QuantConflictFind::effort_conflict ){
+              if (p->atConflictEffort()) {
                 Kind kn = k;
                 if( d_vars[index].getKind()==PLUS ){
                   kn = MINUS;
@@ -1332,7 +1334,7 @@ void MatchGen::reset( QuantConflictFind * p, bool tgt, QuantInfo * qi ) {
       if( d_tgt ){
         success = p->areMatchEqual( nn[0], nn[1] );
       }else{
-        if( p->d_effort==QuantConflictFind::effort_conflict ){
+        if (p->atConflictEffort()) {
           success = p->areDisequal( nn[0], nn[1] );
         }else{
           success = p->areMatchDisequal( nn[0], nn[1] );
@@ -1375,7 +1377,8 @@ void MatchGen::reset( QuantConflictFind * p, bool tgt, QuantInfo * qi ) {
     }else{
       if( d_tgt && d_n.getKind()==FORALL ){
         //fail
-      }else if( d_n.getKind()==FORALL && p->d_effort==QuantConflictFind::effort_conflict && !options::qcfNestedConflict() ){
+      } else if (d_n.getKind() == FORALL && p->atConflictEffort() &&
+                 !options::qcfNestedConflict()) {
         //fail
       }else{
         //reset the first child to d_tgt
@@ -1863,7 +1866,7 @@ void MatchGen::setInvalid() {
 }
 
 bool MatchGen::isHandledBoolConnective( TNode n ) {
-  return TermDb::isBoolConnectiveTerm( n ) && n.getKind()!=SEP_STAR;
+  return TermUtil::isBoolConnectiveTerm( n ) && n.getKind()!=SEP_STAR;
 }
 
 bool MatchGen::isHandledUfTerm( TNode n ) {
@@ -1896,14 +1899,13 @@ bool MatchGen::isHandled( TNode n ) {
   return true;
 }
 
-
-QuantConflictFind::QuantConflictFind( QuantifiersEngine * qe, context::Context* c ) :
-QuantifiersModule( qe ),
-d_conflict( c, false ) {
-  d_fid_count = 0;
-  d_true = NodeManager::currentNM()->mkConst<bool>(true);
-  d_false = NodeManager::currentNM()->mkConst<bool>(false);
-}
+QuantConflictFind::QuantConflictFind(QuantifiersEngine* qe, context::Context* c)
+    : QuantifiersModule(qe),
+      d_conflict(c, false),
+      d_true(NodeManager::currentNM()->mkConst<bool>(true)),
+      d_false(NodeManager::currentNM()->mkConst<bool>(false)),
+      d_effort(EFFORT_INVALID),
+      d_needs_computeRelEqr() {}
 
 Node QuantConflictFind::mkEqNode( Node a, Node b ) {
   return a.eqNode( b );
@@ -1943,16 +1945,6 @@ void QuantConflictFind::registerQuantifier( Node q ) {
   }
 }
 
-short QuantConflictFind::getMaxQcfEffort() {
-  if( options::qcfMode()==QCF_CONFLICT_ONLY ){
-    return effort_conflict;
-  }else if( options::qcfMode()==QCF_PROP_EQ || options::qcfMode()==QCF_PARTIAL ){
-    return effort_prop_eq;
-  }else{
-    return 0;
-  }
-}
-
 bool QuantConflictFind::areMatchEqual( TNode n1, TNode n2 ) {
   //if( d_effort==QuantConflictFind::effort_mc ){
   //  return n1==n2 || !areDisequal( n1, n2 );
@@ -1962,7 +1954,7 @@ bool QuantConflictFind::areMatchEqual( TNode n1, TNode n2 ) {
 }
 
 bool QuantConflictFind::areMatchDisequal( TNode n1, TNode n2 ) {
-  //if( d_effort==QuantConflictFind::effort_conflict ){
+  // if( d_effort==QuantConflictFind::Effort::Conflict ){
   //  return areDisequal( n1, n2 );
   //}else{
   return n1!=n2;
@@ -2028,10 +2020,35 @@ void QuantConflictFind::setIrrelevantFunction( TNode f ) {
   }
 }
 
+namespace {
+
+// Returns the beginning of a range of efforts. The range can be iterated
+// through as unsigned using operator++.
+inline QuantConflictFind::Effort QcfEffortStart() {
+  return QuantConflictFind::EFFORT_CONFLICT;
+}
+
+// Returns the beginning of a range of efforts. The value returned is included
+// in the range.
+inline QuantConflictFind::Effort QcfEffortEnd() {
+  switch (options::qcfMode()) {
+    case QCF_PROP_EQ:
+    case QCF_PARTIAL:
+      return QuantConflictFind::EFFORT_PROP_EQ;
+    case QCF_CONFLICT_ONLY:
+    default:
+      return QuantConflictFind::EFFORT_PROP_EQ;
+  }
+}
+
+}  // namespace
+
 /** check */
-void QuantConflictFind::check( Theory::Effort level, unsigned quant_e ) {
+void QuantConflictFind::check(Theory::Effort level, QEffort quant_e)
+{
   CodeTimer codeTimer(d_quantEngine->d_statistics.d_qcf_time);
-  if( quant_e==QuantifiersEngine::QEFFORT_CONFLICT ){
+  if (quant_e == QEFFORT_CONFLICT)
+  {
     Trace("qcf-check") << "QCF : check : " << level << std::endl;
     if( d_conflict ){
       Trace("qcf-check2") << "QCF : finished check : already in conflict." << std::endl;
@@ -2059,10 +2076,9 @@ void QuantConflictFind::check( Theory::Effort level, unsigned quant_e ) {
         debugPrint("qcf-debug");
         Trace("qcf-debug") << std::endl;
       }
-      short end_e = getMaxQcfEffort();
       bool isConflict = false;
-      for( short e = effort_conflict; e<=end_e; e++ ){
-        d_effort = e;
+      for (unsigned e = QcfEffortStart(), end = QcfEffortEnd(); e <= end; ++e) {
+        d_effort = static_cast<Effort>(e);
         Trace("qcf-check") << "Checking quantified formulas at effort " << e << "..." << std::endl;
         for( unsigned i=0; i<d_quantEngine->getModel()->getNumAssertedQuantifiers(); i++ ){
           Node q = d_quantEngine->getModel()->getAssertedQuantifier( i, true );
@@ -2097,18 +2113,22 @@ void QuantConflictFind::check( Theory::Effort level, unsigned quant_e ) {
                         if( !tcs ){
                           //for debugging
                           if( Debug.isOn("qcf-check-inst") ){
-                            Node inst = d_quantEngine->getInstantiation( q, terms );
+                            Node inst = d_quantEngine->getInstantiate()
+                                            ->getInstantiation(q, terms);
                             Debug("qcf-check-inst") << "Check instantiation " << inst << "..." << std::endl;
                             Assert( !getTermDatabase()->isEntailed( inst, true ) );
-                            Assert( getTermDatabase()->isEntailed( inst, false ) || e>effort_conflict );
+                            Assert(getTermDatabase()->isEntailed(inst, false) ||
+                                   e > EFFORT_CONFLICT);
                           }
-                          if( d_quantEngine->addInstantiation( q, terms ) ){
+                          if (d_quantEngine->getInstantiate()->addInstantiation(
+                                  q, terms))
+                          {
                             Trace("qcf-check") << "   ... Added instantiation" << std::endl;
                             Trace("qcf-inst") << "*** Was from effort " << e << " : " << std::endl;
                             qi->debugPrintMatch("qcf-inst");
                             Trace("qcf-inst") << std::endl;
                             ++addedLemmas;
-                            if( e==effort_conflict ){
+                            if (e == EFFORT_CONFLICT) {
                               d_quantEngine->markRelevant( q );
                               ++(d_quantEngine->d_statistics.d_instantiations_qcf);
                               if( options::qcfAllConflict() ){
@@ -2117,7 +2137,7 @@ void QuantConflictFind::check( Theory::Effort level, unsigned quant_e ) {
                                 d_conflict.set( true );
                               }
                               break;
-                            }else if( e==effort_prop_eq ){
+                            } else if (e == EFFORT_PROP_EQ) {
                               d_quantEngine->markRelevant( q );
                               ++(d_quantEngine->d_statistics.d_instantiations_qcf);
                             }
@@ -2160,7 +2180,11 @@ void QuantConflictFind::check( Theory::Effort level, unsigned quant_e ) {
         double clSet2 = double(clock())/double(CLOCKS_PER_SEC);
         Trace("qcf-engine") << "Finished conflict find engine, time = " << (clSet2-clSet);
         if( addedLemmas>0 ){
-          Trace("qcf-engine") << ", effort = " << ( d_effort==effort_conflict ? "conflict" : ( d_effort==effort_prop_eq ? "prop_eq" : "mc" ) );
+          Trace("qcf-engine")
+              << ", effort = "
+              << (d_effort == EFFORT_CONFLICT
+                      ? "conflict"
+                      : (d_effort == EFFORT_PROP_EQ ? "prop_eq" : "mc"));
           Trace("qcf-engine") << ", addedLemmas = " << addedLemmas;
         }
         Trace("qcf-engine") << std::endl;
@@ -2185,7 +2209,7 @@ void QuantConflictFind::computeRelevantEqr() {
       Node r = (*eqcs_i);
       if( getTermDatabase()->hasTermCurrent( r ) ){
         TypeNode rtn = r.getType();
-        if( !options::cbqi() || !TermDb::hasInstConstAttr( r ) ){
+        if( !options::cbqi() || !TermUtil::hasInstConstAttr( r ) ){
           d_eqcs[rtn].push_back( r );
         }
       }
@@ -2272,6 +2296,21 @@ TNode QuantConflictFind::getZero( Kind k ) {
   }
 }
 
+std::ostream& operator<<(std::ostream& os, const QuantConflictFind::Effort& e) {
+  switch (e) {
+    case QuantConflictFind::EFFORT_INVALID:
+      os << "Invalid";
+      break;
+    case QuantConflictFind::EFFORT_CONFLICT:
+      os << "Conflict";
+      break;
+    case QuantConflictFind::EFFORT_PROP_EQ:
+      os << "PropEq";
+      break;
+  }
+  return os;
+}
+
 } /* namespace CVC4::theory::quantifiers */
 } /* namespace CVC4::theory */
 } /* namespace CVC4 */
