@@ -2,9 +2,9 @@
 /*! \file theory_quantifiers_type_rules.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Andrew Reynolds, Morgan Deters, Paul Meng
+ **   Morgan Deters, Andrew Reynolds, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -27,7 +27,7 @@ namespace quantifiers {
 
 struct QuantifierForallTypeRule {
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Debug("typecheck-q") << "type check for fa " << n << std::endl;
     Assert(n.getKind() == kind::FORALL && n.getNumChildren()>0 );
     if( check ){
@@ -47,7 +47,7 @@ struct QuantifierForallTypeRule {
 
 struct QuantifierExistsTypeRule {
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Debug("typecheck-q") << "type check for ex " << n << std::endl;
     Assert(n.getKind() == kind::EXISTS && n.getNumChildren()>0 );
     if( check ){
@@ -67,7 +67,7 @@ struct QuantifierExistsTypeRule {
 
 struct QuantifierBoundVarListTypeRule {
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Assert(n.getKind() == kind::BOUND_VAR_LIST );
     if( check ){
       for( int i=0; i<(int)n.getNumChildren(); i++ ){
@@ -82,11 +82,12 @@ struct QuantifierBoundVarListTypeRule {
 
 struct QuantifierInstPatternTypeRule {
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Assert(n.getKind() == kind::INST_PATTERN );
     if( check ){
       TypeNode tn = n[0].getType(check);
-      if( tn.isFunction() ){
+      // this check catches the common mistake writing :pattern (f x) instead of :pattern ((f x))
+      if( n[0].isVar() && n[0].getKind()!=kind::BOUND_VARIABLE && tn.isFunction() ){
         throw TypeCheckingExceptionPrivate(n[0], "Pattern must be a list of fully-applied terms.");
       }
     }
@@ -96,7 +97,7 @@ struct QuantifierInstPatternTypeRule {
 
 struct QuantifierInstNoPatternTypeRule {
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Assert(n.getKind() == kind::INST_NO_PATTERN );
     return nodeManager->instPatternType();
   }
@@ -104,7 +105,7 @@ struct QuantifierInstNoPatternTypeRule {
 
 struct QuantifierInstAttributeTypeRule {
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Assert(n.getKind() == kind::INST_ATTRIBUTE );
     return nodeManager->instPatternType();
   }
@@ -112,7 +113,7 @@ struct QuantifierInstAttributeTypeRule {
 
 struct QuantifierInstPatternListTypeRule {
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Assert(n.getKind() == kind::INST_PATTERN_LIST );
     if( check ){
       for( int i=0; i<(int)n.getNumChildren(); i++ ){
@@ -127,7 +128,7 @@ struct QuantifierInstPatternListTypeRule {
 
 struct QuantifierInstClosureTypeRule {
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Assert(n.getKind() == kind::INST_CLOSURE );
     if( check ){
       TypeNode tn = n[0].getType(check);
@@ -154,7 +155,7 @@ public:
    */
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n,
                                      bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Debug("typecheck-r") << "type check for rr " << n << std::endl;
     Assert(n.getKind() == kind::REWRITE_RULE && n.getNumChildren()==3 );
     if( check ){
@@ -180,7 +181,7 @@ class RRRewriteTypeRule {
 public:
 
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Assert(n.getKind() == kind::RR_REWRITE );
     if( check ){
       if( n[0].getType(check)!=n[1].getType(check) ){
@@ -202,7 +203,7 @@ class RRRedDedTypeRule {
 public:
 
   inline static TypeNode computeType(NodeManager* nodeManager, TNode n, bool check)
-    throw(TypeCheckingExceptionPrivate) {
+  {
     Assert(n.getKind() == kind::RR_REDUCTION ||
            n.getKind() == kind::RR_DEDUCTION );
     if( check ){
