@@ -2,9 +2,9 @@
 /*! \file theory_sets_rewriter.cpp
  ** \verbatim
  ** Top contributors (to current version):
- **   Andrew Reynolds, Paul Meng, Kshitij Bansal
+ **   Kshitij Bansal, Andrew Reynolds, Paul Meng
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -15,17 +15,18 @@
  **/
 
 #include "theory/sets/theory_sets_rewriter.h"
-#include "theory/sets/normal_form.h"
-#include "theory/sets/rels_utils.h"
+
 #include "expr/attribute.h"
 #include "options/sets_options.h"
+#include "theory/sets/normal_form.h"
+#include "theory/sets/rels_utils.h"
 
 namespace CVC4 {
 namespace theory {
 namespace sets {
 
 typedef std::set<TNode> Elements;
-typedef std::hash_map<TNode, Elements, TNodeHashFunction> SettermElementsMap;
+typedef std::unordered_map<TNode, Elements, TNodeHashFunction> SettermElementsMap;
 
 struct FlattenedNodeTag {};
 typedef expr::Attribute<FlattenedNodeTag, bool> flattened;
@@ -50,7 +51,7 @@ RewriteResponse flattenNode(TNode n, TNode trivialNode, TNode skipNode)
     return RewriteResponse(REWRITE_DONE, n);
   }
 
-  typedef std::hash_set<TNode, TNodeHashFunction> node_set;
+  typedef std::unordered_set<TNode, TNodeHashFunction> node_set;
 
   node_set visited;
   visited.insert(skipNode);
@@ -92,7 +93,7 @@ RewriteResponse flattenNode(TNode n, TNode trivialNode, TNode skipNode)
   static const unsigned MAX_CHILDREN = (1u << __CVC4__EXPR__NODE_VALUE__NBITS__NCHILDREN ) - 1;
   AlwaysAssert(childList.size() < MAX_CHILDREN, "do not support formulas this big");
 
-  ChildList::iterator cur = childList.begin(), next, en = childList.end();
+  ChildList::iterator cur = childList.begin(), en = childList.end();
   Node ret = (*cur);
   ++cur;
   while( cur != en ) {
