@@ -4,7 +4,7 @@
  ** Top contributors (to current version):
  **   Dejan Jovanovic, Andrew Reynolds, Morgan Deters
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -35,15 +35,17 @@ bool PreRegisterVisitor::alreadyVisited(TNode current, TNode parent) {
 
   Debug("register::internal") << "PreRegisterVisitor::alreadyVisited(" << current << "," << parent << ")" << std::endl;
 
-  if( ( parent.getKind() == kind::FORALL ||
-        parent.getKind() == kind::EXISTS ||
-        parent.getKind() == kind::REWRITE_RULE ||
-        parent.getKind() == kind::SEP_STAR ||
-        parent.getKind() == kind::SEP_WAND ||
-        ( parent.getKind() == kind::SEP_LABEL && current.getType().isBoolean() )
-        // parent.getKind() == kind::CARDINALITY_CONSTRAINT
-      ) &&
-      current != parent ) {
+  if ((parent.getKind() == kind::FORALL || parent.getKind() == kind::EXISTS
+       || parent.getKind() == kind::LAMBDA
+       || parent.getKind() == kind::CHOICE
+       || parent.getKind() == kind::REWRITE_RULE
+       || parent.getKind() == kind::SEP_STAR
+       || parent.getKind() == kind::SEP_WAND
+       || (parent.getKind() == kind::SEP_LABEL && current.getType().isBoolean())
+       // parent.getKind() == kind::CARDINALITY_CONSTRAINT
+       )
+      && current != parent)
+  {
     Debug("register::internal") << "quantifier:true" << std::endl;
     return true;
   }
@@ -179,15 +181,17 @@ bool SharedTermsVisitor::alreadyVisited(TNode current, TNode parent) const {
 
   Debug("register::internal") << "SharedTermsVisitor::alreadyVisited(" << current << "," << parent << ")" << std::endl;
 
-  if( ( parent.getKind() == kind::FORALL ||
-        parent.getKind() == kind::EXISTS ||
-        parent.getKind() == kind::REWRITE_RULE ||
-        parent.getKind() == kind::SEP_STAR ||
-        parent.getKind() == kind::SEP_WAND ||
-        ( parent.getKind() == kind::SEP_LABEL && current.getType().isBoolean() )
-        // parent.getKind() == kind::CARDINALITY_CONSTRAINT
-      ) &&
-      current != parent ) {
+  if ((parent.getKind() == kind::FORALL || parent.getKind() == kind::EXISTS
+       || parent.getKind() == kind::LAMBDA
+       || parent.getKind() == kind::CHOICE
+       || parent.getKind() == kind::REWRITE_RULE
+       || parent.getKind() == kind::SEP_STAR
+       || parent.getKind() == kind::SEP_WAND
+       || (parent.getKind() == kind::SEP_LABEL && current.getType().isBoolean())
+       // parent.getKind() == kind::CARDINALITY_CONSTRAINT
+       )
+      && current != parent)
+  {
     Debug("register::internal") << "quantifier:true" << std::endl;
     return true;
   }
