@@ -4,7 +4,7 @@
  ** Top contributors (to current version):
  **   Tim King, Andrew Reynolds, Dejan Jovanovic
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -51,28 +51,31 @@ std::ostream& operator<<(std::ostream& os, Theory::Effort level){
   return os;
 }/* ostream& operator<<(ostream&, Theory::Effort) */
 
-
-Theory::Theory(TheoryId id, context::Context* satContext,
-               context::UserContext* userContext, OutputChannel& out,
-               Valuation valuation, const LogicInfo& logicInfo,
-               std::string name) throw()
-    : d_id(id)
-    , d_instanceName(name)
-    , d_satContext(satContext)
-    , d_userContext(userContext)
-    , d_logicInfo(logicInfo)
-    , d_facts(satContext)
-    , d_factsHead(satContext, 0)
-    , d_sharedTermsIndex(satContext, 0)
-    , d_careGraph(NULL)
-    , d_quantEngine(NULL)
-    , d_extTheory(NULL)
-    , d_checkTime(getFullInstanceName() + "::checkTime")
-    , d_computeCareGraphTime(getFullInstanceName() + "::computeCareGraphTime")
-    , d_sharedTerms(satContext)
-    , d_out(&out)
-    , d_valuation(valuation)
-    , d_proofsEnabled(false)
+Theory::Theory(TheoryId id,
+               context::Context* satContext,
+               context::UserContext* userContext,
+               OutputChannel& out,
+               Valuation valuation,
+               const LogicInfo& logicInfo,
+               std::string name)
+    : d_id(id),
+      d_instanceName(name),
+      d_satContext(satContext),
+      d_userContext(userContext),
+      d_logicInfo(logicInfo),
+      d_facts(satContext),
+      d_factsHead(satContext, 0),
+      d_sharedTermsIndex(satContext, 0),
+      d_careGraph(NULL),
+      d_quantEngine(NULL),
+      d_extTheory(NULL),
+      d_checkTime(getStatsPrefix(id) + name + "::checkTime"),
+      d_computeCareGraphTime(getStatsPrefix(id) + name
+                             + "::computeCareGraphTime"),
+      d_sharedTerms(satContext),
+      d_out(&out),
+      d_valuation(valuation),
+      d_proofsEnabled(false)
 {
   smtStatisticsRegistry()->registerStat(&d_checkTime);
   smtStatisticsRegistry()->registerStat(&d_computeCareGraphTime);
@@ -220,8 +223,8 @@ void Theory::debugPrintFacts() const{
   printFacts(DebugChannel.getStream());
 }
 
-std::hash_set<TNode, TNodeHashFunction> Theory::currentlySharedTerms() const{
-  std::hash_set<TNode, TNodeHashFunction> currentlyShared;
+std::unordered_set<TNode, TNodeHashFunction> Theory::currentlySharedTerms() const{
+  std::unordered_set<TNode, TNodeHashFunction> currentlyShared;
   for (shared_terms_iterator i = shared_terms_begin(),
            i_end = shared_terms_end(); i != i_end; ++i) {
     currentlyShared.insert (*i);
@@ -335,12 +338,6 @@ EntailmentCheckParameters::EntailmentCheckParameters(TheoryId tid)
   : d_tid(tid) {
 }
 
-std::string Theory::getFullInstanceName() const {
-  std::stringstream ss;
-  ss << "theory<" << d_id << ">" << d_instanceName;
-  return ss.str();
-}
-
 EntailmentCheckParameters::~EntailmentCheckParameters(){}
 
 TheoryId EntailmentCheckParameters::getTheoryId() const {
