@@ -2,9 +2,9 @@
 /*! \file type_enumerator.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Andrew Reynolds, Tim King
+ **   Morgan Deters, Tim King, Andrew Reynolds
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -28,39 +28,41 @@ namespace CVC4 {
 namespace theory {
 
 class NoMoreValuesException : public Exception {
-public:
-  NoMoreValuesException(TypeNode n) throw() :
-    Exception("No more values for type `" + n.toString() + "'") {
+ public:
+  NoMoreValuesException(TypeNode n)
+      : Exception("No more values for type `" + n.toString() + "'")
+  {
   }
 };/* class NoMoreValuesException */
 
 class TypeEnumeratorInterface {
-  TypeNode d_type;
-
-public:
-
-  TypeEnumeratorInterface(TypeNode type) :
-    d_type(type) {
-  }
+ public:
+  TypeEnumeratorInterface(TypeNode type) : d_type(type) {}
 
   virtual ~TypeEnumeratorInterface() {}
 
   /** Is this enumerator out of constants to enumerate? */
-  virtual bool isFinished() throw() = 0;
+  virtual bool isFinished() = 0;
 
-  /** Get the current constant of this type (throws if no such constant exists) */
+  /**
+   * Get the current constant of this type.
+   *
+   * @throws NoMoreValuesException if no such constant exists.
+   */
   virtual Node operator*() = 0;
 
-  /** Increment the pointer to the next available constant */
+  /** Increment the pointer to the next available constant. */
   virtual TypeEnumeratorInterface& operator++() = 0;
 
-  /** Clone this enumerator */
+  /** Clone this enumerator. */
   virtual TypeEnumeratorInterface* clone() const = 0;
 
-  /** Get the type from which we're enumerating constants */
-  TypeNode getType() const throw() { return d_type; }
+  /** Get the type from which we're enumerating constants. */
+  TypeNode getType() const { return d_type; }
 
-};/* class TypeEnumeratorInterface */
+ private:
+  const TypeNode d_type;
+}; /* class TypeEnumeratorInterface */
 
 // AJR: This class stores particular information that is relevant to type enumeration.
 //      For finite model finding, we set d_fixed_usort=true,
@@ -82,20 +84,28 @@ public:
     TypeEnumeratorInterface(type) {
   }
 
-  TypeEnumeratorInterface* clone() const { return new T(static_cast<const T&>(*this)); }
+  TypeEnumeratorInterface* clone() const override
+  {
+    return new T(static_cast<const T&>(*this));
+  }
 
 };/* class TypeEnumeratorBase */
 
+/** Type enumerator class.
+ * Enumerates values for a type.
+ * Its constructor takes the type to enumerate and a pointer to a
+ * TypeEnumeratorProperties class, which this type enumerator does not own.
+ */
 class TypeEnumerator {
   TypeEnumeratorInterface* d_te;
 
-  static TypeEnumeratorInterface* mkTypeEnumerator(TypeNode type, TypeEnumeratorProperties * tep)
-    throw(AssertionException);
+  static TypeEnumeratorInterface* mkTypeEnumerator(
+      TypeNode type, TypeEnumeratorProperties* tep);
 
-public:
-
-  TypeEnumerator(TypeNode type, TypeEnumeratorProperties * tep = NULL) throw() :
-    d_te(mkTypeEnumerator(type, tep)) {
+ public:
+  TypeEnumerator(TypeNode type, TypeEnumeratorProperties* tep = nullptr)
+      : d_te(mkTypeEnumerator(type, tep))
+  {
   }
 
   TypeEnumerator(const TypeEnumerator& te) :
@@ -110,8 +120,8 @@ public:
   }
 
   ~TypeEnumerator() { delete d_te; }
-
-  bool isFinished() throw() {
+  bool isFinished()
+  {
 // On Mac clang, there appears to be a code generation bug in an exception
 // block here.  For now, there doesn't appear a good workaround; just disable
 // assertions on that setup.
@@ -138,7 +148,8 @@ public:
 #endif /* CVC4_ASSERTIONS && !(APPLE || clang) */
     return d_te->isFinished();
   }
-  Node operator*() throw(NoMoreValuesException) {
+  Node operator*()
+  {
 // On Mac clang, there appears to be a code generation bug in an exception
 // block above (and perhaps here, too).  For now, there doesn't appear a
 // good workaround; just disable assertions on that setup.
@@ -156,11 +167,19 @@ public:
     return **d_te;
 #endif /* CVC4_ASSERTIONS && !(APPLE || clang) */
   }
-  TypeEnumerator& operator++() throw() { ++*d_te; return *this; }
-  TypeEnumerator operator++(int) throw() { TypeEnumerator te = *this; ++*d_te; return te; }
-
-  TypeNode getType() const throw() { return d_te->getType(); }
+  TypeEnumerator& operator++()
+  {
+    ++*d_te;
+    return *this;
+  }
+  TypeEnumerator operator++(int)
+  {
+    TypeEnumerator te = *this;
+    ++*d_te;
+    return te;
+  }
 
+  TypeNode getType() const { return d_te->getType(); }
 };/* class TypeEnumerator */
 
 }/* CVC4::theory namespace */
