@@ -2,9 +2,9 @@
 /*! \file equality_engine_types.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Dejan Jovanovic, Andrew Reynolds, Paul Meng
+ **   Dejan Jovanovic, Andres Noetzli, Andrew Reynolds
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -17,10 +17,15 @@
 
 #include "cvc4_private.h"
 
+#ifndef __CVC4__THEORY__UF__EQUALITY_ENGINE_TYPES_H
+#define __CVC4__THEORY__UF__EQUALITY_ENGINE_TYPES_H
+
 #include <string>
 #include <iostream>
 #include <sstream>
 
+#include "util/hash.h"
+
 namespace CVC4 {
 namespace theory {
 namespace eq {
@@ -264,15 +269,8 @@ public:
 
 /** A pair of ids */
 typedef std::pair<EqualityNodeId, EqualityNodeId> EqualityPair;
-
-struct EqualityPairHashFunction {
-  size_t operator () (const EqualityPair& pair) const {
-    size_t hash = 0;
-    hash = 0x9e3779b9 + pair.first;
-    hash ^= 0x9e3779b9 + pair.second + (hash << 6) + (hash >> 2);
-    return hash;
-  }
-};
+using EqualityPairHashFunction =
+    PairHashFunction<EqualityNodeId, EqualityNodeId>;
 
 enum FunctionApplicationType {
   /** This application is an equality a = b */
@@ -353,12 +351,13 @@ struct TriggerInfo {
   Node trigger;
   /** Polarity of the trigger */
   bool polarity;
-  TriggerInfo() {}
+  TriggerInfo() : polarity(false) {}
   TriggerInfo(Node trigger, bool polarity)
-  : trigger(trigger), polarity(polarity) {}
-  };
-
+      : trigger(trigger), polarity(polarity) {}
+};
 
 } // namespace eq
 } // namespace theory
 } // namespace CVC4
+
+#endif /* __CVC4__THEORY__UF__EQUALITY_ENGINE_TYPES_H */
