@@ -2,9 +2,9 @@
 /*! \file theory_uf.cpp
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Andrew Reynolds, Dejan Jovanovic
+ **   Andrew Reynolds, Morgan Deters, Dejan Jovanovic
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -17,6 +17,8 @@
 
 #include "theory/uf/theory_uf.h"
 
+#include <memory>
+
 #include "options/quantifiers_options.h"
 #include "options/smt_options.h"
 #include "options/uf_options.h"
@@ -28,6 +30,7 @@
 #include "theory/uf/theory_uf_strong_solver.h"
 #include "theory/quantifiers/term_database.h"
 #include "options/theory_options.h"
+#include "theory/uf/theory_uf_rewriter.h"
 
 using namespace std;
 
@@ -36,24 +39,31 @@ namespace theory {
 namespace uf {
 
 /** Constructs a new instance of TheoryUF w.r.t. the provided context.*/
-TheoryUF::TheoryUF(context::Context* c, context::UserContext* u,
-                   OutputChannel& out, Valuation valuation,
-                   const LogicInfo& logicInfo, std::string instanceName)
+TheoryUF::TheoryUF(context::Context* c,
+                   context::UserContext* u,
+                   OutputChannel& out,
+                   Valuation valuation,
+                   const LogicInfo& logicInfo,
+                   std::string instanceName)
     : Theory(THEORY_UF, c, u, out, valuation, logicInfo, instanceName),
       d_notify(*this),
       /* The strong theory solver can be notified by EqualityEngine::init(),
        * so make sure it's initialized first. */
       d_thss(NULL),
-      d_equalityEngine(d_notify, c, instanceName + "theory::uf::TheoryUF",
-                       true),
+      d_equalityEngine(d_notify, c, instanceName + "theory::uf::ee", true),
       d_conflict(c, false),
-      d_literalsToPropagate(c),
-      d_literalsToPropagateIndex(c, 0),
+      d_extensionality(u),
+      d_uf_std_skolem(u),
       d_functionsTerms(c),
       d_symb(u, instanceName)
 {
+  d_true = NodeManager::currentNM()->mkConst( true );
+
   // The kinds we are treating as function application in congruence
-  d_equalityEngine.addFunctionKind(kind::APPLY_UF);
+  d_equalityEngine.addFunctionKind(kind::APPLY_UF, false, options::ufHo());
+  if( options::ufHo() ){
+    d_equalityEngine.addFunctionKind(kind::HO_APPLY);
+  }
 }
 
 TheoryUF::~TheoryUF() {
@@ -65,6 +75,10 @@ void TheoryUF::setMasterEqualityEngine(eq::EqualityEngine* eq) {
 }
 
 void TheoryUF::finishInit() {
+  // combined cardinality constraints are not evaluated in getModelValue
+  TheoryModel* tm = d_valuation.getModel();
+  Assert(tm != nullptr);
+  tm->setUnevaluatedKind(kind::COMBINED_CARDINALITY_CONSTRAINT);
   // initialize the strong solver
   if (options::finiteModelFind() && options::ufssMode()!=UF_SS_NONE) {
     d_thss = new StrongSolverTheoryUF(getSatContext(), getUserContext(), *d_out, this);
@@ -123,15 +137,25 @@ void TheoryUF::check(Effort level) {
     TNode atom = polarity ? fact : fact[0];
     if (atom.getKind() == kind::EQUAL) {
       d_equalityEngine.assertEquality(atom, polarity, fact);
+      if( options::ufHo() && options::ufHoExt() ){
+        if( !polarity && !d_conflict && atom[0].getType().isFunction() ){
+          applyExtensionality( fact );
+        }
+      }
     } else if (atom.getKind() == kind::CARDINALITY_CONSTRAINT || atom.getKind() == kind::COMBINED_CARDINALITY_CONSTRAINT) {
       if( d_thss == NULL ){
-        std::stringstream ss;
-        ss << "Cardinality constraint " << atom << " was asserted, but the logic does not allow it." << std::endl;
-        ss << "Try using a logic containing \"UFC\"." << std::endl;
-        throw Exception( ss.str() );
+        if( !getLogicInfo().hasCardinalityConstraints() ){
+          std::stringstream ss;
+          ss << "Cardinality constraint " << atom << " was asserted, but the logic does not allow it." << std::endl;
+          ss << "Try using a logic containing \"UFC\"." << std::endl;
+          throw Exception( ss.str() );
+        }else{
+          // support for cardinality constraints is not enabled, set incomplete
+          d_out->setIncomplete();
+        } 
       }
       //needed for models
-      if( options::produceModels() && ( atom.getKind() == kind::COMBINED_CARDINALITY_CONSTRAINT || options::ufssMode()!=UF_SS_FULL ) ){
+      if( options::produceModels() ){
         d_equalityEngine.assertPredicate(atom, polarity, fact);
       }
     } else {
@@ -146,9 +170,71 @@ void TheoryUF::check(Effort level) {
         d_conflict = true;
       }
     }
+    if(! d_conflict && fullEffort(level) ){
+      if( options::ufHo() ){
+        checkHigherOrder();
+      }
+    }
   }
 }/* TheoryUF::check() */
 
+Node TheoryUF::getApplyUfForHoApply( Node node ) {
+  Assert( node[0].getType().getNumChildren()==2 );
+  std::vector< TNode > args;
+  Node f = TheoryUfRewriter::decomposeHoApply( node, args, true );
+  Node new_f = f;
+  if( !TheoryUfRewriter::canUseAsApplyUfOperator( f ) ){
+    NodeNodeMap::const_iterator itus = d_uf_std_skolem.find( f );
+    if( itus==d_uf_std_skolem.end() ){
+      // introduce skolem to make a standard APPLY_UF
+      new_f = NodeManager::currentNM()->mkSkolem( "app_uf", f.getType() );
+      Node lem = new_f.eqNode( f );
+      Trace("uf-ho-lemma") << "uf-ho-lemma : Skolem definition for apply-conversion : " << lem << std::endl;
+      d_out->lemma( lem );
+      d_uf_std_skolem[f] = new_f;
+    }else{
+      new_f = (*itus).second;
+    }
+  }
+  Assert( TheoryUfRewriter::canUseAsApplyUfOperator( new_f ) );
+  args[0] = new_f;
+  Node ret = NodeManager::currentNM()->mkNode( kind::APPLY_UF, args );
+  return ret;
+}
+
+Node TheoryUF::getOperatorForApplyTerm( TNode node ) {
+  Assert( node.getKind()==kind::APPLY_UF || node.getKind()==kind::HO_APPLY );
+  if( node.getKind()==kind::APPLY_UF ){
+    return node.getOperator();
+  }else{
+    return d_equalityEngine.getRepresentative( node[0] );
+  }
+}
+
+unsigned TheoryUF::getArgumentStartIndexForApplyTerm( TNode node ) {
+  Assert( node.getKind()==kind::APPLY_UF || node.getKind()==kind::HO_APPLY );
+  return node.getKind()==kind::APPLY_UF ? 0 : 1;
+}
+
+Node TheoryUF::expandDefinition(LogicRequest &logicRequest, Node node) {
+  Trace("uf-ho-debug") << "uf-ho-debug : expanding definition : " << node << std::endl;
+  if( node.getKind()==kind::HO_APPLY ){
+    if( !options::ufHo() ){
+      std::stringstream ss;
+      ss << "Partial function applications are not supported in default mode, try --uf-ho.";
+      throw LogicException(ss.str());
+    }
+    // convert HO_APPLY to APPLY_UF if fully applied
+    if( node[0].getType().getNumChildren()==2 ){
+      Trace("uf-ho") << "uf-ho : expanding definition : " << node << std::endl;
+      Node ret = getApplyUfForHoApply( node );
+      Trace("uf-ho") << "uf-ho : expandDefinition : " << node << " to " << ret << std::endl;
+      return ret;
+    }
+  }
+  return node;
+}
+
 void TheoryUF::preRegisterTerm(TNode node) {
   Debug("uf") << "TheoryUF::preRegisterTerm(" << node << ")" << std::endl;
 
@@ -156,12 +242,17 @@ void TheoryUF::preRegisterTerm(TNode node) {
     d_thss->preRegisterTerm(node);
   }
 
+  // we always use APPLY_UF if not higher-order, HO_APPLY if higher-order
+  //Assert( node.getKind()!=kind::APPLY_UF || !options::ufHo() );
+  Assert( node.getKind()!=kind::HO_APPLY || options::ufHo() );
+
   switch (node.getKind()) {
   case kind::EQUAL:
     // Add the trigger for equality
     d_equalityEngine.addTriggerEquality(node);
     break;
   case kind::APPLY_UF:
+  case kind::HO_APPLY:
     // Maybe it's a predicate
     if (node.getType().isBoolean()) {
       // Get triggered for both equal and dis-equal
@@ -245,34 +336,52 @@ Node TheoryUF::explain(TNode literal, eq::EqProof* pf) {
   return mkAnd(assumptions);
 }
 
-void TheoryUF::collectModelInfo( TheoryModel* m ){
+bool TheoryUF::collectModelInfo(TheoryModel* m)
+{
+  Debug("uf") << "UF : collectModelInfo " << std::endl;
   set<Node> termSet;
 
   // Compute terms appearing in assertions and shared terms
   computeRelevantTerms(termSet);
 
-  m->assertEqualityEngine( &d_equalityEngine, &termSet );
-  // if( fullModel ){
-  //   std::map< TypeNode, TypeEnumerator* > type_enums;
-  //   //must choose proper representatives
-  //   // for each equivalence class, specify fresh constant as representative
-  //   eq::EqClassesIterator eqcs_i = eq::EqClassesIterator( &d_equalityEngine );
-  //   while( !eqcs_i.isFinished() ){
-  //     Node eqc = (*eqcs_i);
-  //     TypeNode tn = eqc.getType();
-  //     if( tn.isSort() ){
-  //       if( type_enums.find( tn )==type_enums.end() ){
-  //         type_enums[tn] = new TypeEnumerator( tn );
-  //       }
-  //       Node rep = *(*type_enums[tn]);
-  //       ++(*type_enums[tn]);
-  //       //specify the constant as the representative
-  //       m->assertEquality( eqc, rep, true );
-  //       m->assertRepresentative( rep );
-  //     }
-  //     ++eqcs_i;
-  //   }
-  // }
+  if (!m->assertEqualityEngine(&d_equalityEngine, &termSet))
+  {
+    return false;
+  }
+
+  if( options::ufHo() ){
+    for( std::set<Node>::iterator it = termSet.begin(); it != termSet.end(); ++it ){
+      Node n = *it;
+      // for model-building with ufHo, we require that APPLY_UF is always
+      // expanded to HO_APPLY
+      if (!collectModelInfoHoTerm(n, m))
+      {
+        return false;
+      }
+    }
+    // must add extensionality disequalities for all pairs of (non-disequal)
+    // function equivalence classes.
+    if (checkExtensionality(m) != 0)
+    {
+      return false;
+    }
+  }
+
+  Debug("uf") << "UF : finish collectModelInfo " << std::endl;
+  return true;
+}
+
+bool TheoryUF::collectModelInfoHoTerm(Node n, TheoryModel* m)
+{
+  if (n.getKind() == kind::APPLY_UF)
+  {
+    Node hn = TheoryUfRewriter::getHoApplyForApplyUf(n);
+    if (!m->assertEquality(n, hn, true))
+    {
+      return false;
+    }
+  }
+  return true;
 }
 
 void TheoryUF::presolve() {
@@ -300,7 +409,7 @@ void TheoryUF::ppStaticLearn(TNode n, NodeBuilder<>& learned) {
 
   vector<TNode> workList;
   workList.push_back(n);
-  __gnu_cxx::hash_set<TNode, TNodeHashFunction> processed;
+  std::unordered_set<TNode, TNodeHashFunction> processed;
 
   while(!workList.empty()) {
     n = workList.back();
@@ -461,7 +570,8 @@ void TheoryUF::addCarePairs( quantifiers::TermArgTrie * t1, quantifiers::TermArg
       if( !d_equalityEngine.areEqual( f1, f2 ) ){
         Debug("uf::sharing") << "TheoryUf::computeCareGraph(): checking function " << f1 << " and " << f2 << std::endl;
         vector< pair<TNode, TNode> > currentPairs;
-        for (unsigned k = 0; k < f1.getNumChildren(); ++ k) {
+        unsigned arg_start_index = getArgumentStartIndexForApplyTerm( f1 );
+        for (unsigned k = arg_start_index; k < f1.getNumChildren(); ++ k) {
           TNode x = f1[k];
           TNode y = f2[k];
           Assert( d_equalityEngine.hasTerm(x) );
@@ -527,17 +637,18 @@ void TheoryUF::computeCareGraph() {
     unsigned functionTerms = d_functionsTerms.size();
     for (unsigned i = 0; i < functionTerms; ++ i) {
       TNode f1 = d_functionsTerms[i];
-      Node op = f1.getOperator();
+      Node op = getOperatorForApplyTerm( f1 );
+      unsigned arg_start_index = getArgumentStartIndexForApplyTerm( f1 );
       std::vector< TNode > reps;
       bool has_trigger_arg = false;
-      for( unsigned j=0; j<f1.getNumChildren(); j++ ){
+      for( unsigned j=arg_start_index; j<f1.getNumChildren(); j++ ){
         reps.push_back( d_equalityEngine.getRepresentative( f1[j] ) );
         if( d_equalityEngine.isTriggerTerm( f1[j], THEORY_UF ) ){
           has_trigger_arg = true;
         }
       }
       if( has_trigger_arg ){
-        index[op].addTerm( f1, reps );
+        index[op].addTerm( f1, reps, arg_start_index );
         arity[op] = reps.size();
       }
     }
@@ -546,14 +657,16 @@ void TheoryUF::computeCareGraph() {
       Debug("uf::sharing") << "TheoryUf::computeCareGraph(): Process index " << itii->first << "..." << std::endl;
       addCarePairs( &itii->second, NULL, arity[ itii->first ], 0 );
     }
+    Debug("uf::sharing") << "TheoryUf::computeCareGraph(): finished." << std::endl;
   }
 }/* TheoryUF::computeCareGraph() */
 
 void TheoryUF::conflict(TNode a, TNode b) {
-  eq::EqProof* pf = d_proofsEnabled ? new eq::EqProof() : NULL;
-  d_conflictNode = explain(a.eqNode(b),pf);
-  ProofUF* puf = d_proofsEnabled ? new ProofUF( pf ) : NULL;
-  d_out->conflict(d_conflictNode, puf);
+  std::shared_ptr<eq::EqProof> pf =
+      d_proofsEnabled ? std::make_shared<eq::EqProof>() : nullptr;
+  d_conflictNode = explain(a.eqNode(b), pf.get());
+  std::unique_ptr<ProofUF> puf(d_proofsEnabled ? new ProofUF(pf) : nullptr);
+  d_out->conflict(d_conflictNode, std::move(puf));
   d_conflict = true;
 }
 
@@ -581,6 +694,235 @@ void TheoryUF::eqNotifyDisequal(TNode t1, TNode t2, TNode reason) {
   }
 }
 
+Node TheoryUF::getExtensionalityDeq(TNode deq)
+{
+  Assert( deq.getKind()==kind::NOT && deq[0].getKind()==kind::EQUAL );
+  Assert( deq[0][0].getType().isFunction() );
+  std::map<Node, Node>::iterator it = d_extensionality_deq.find(deq);
+  if (it == d_extensionality_deq.end())
+  {
+    TypeNode tn = deq[0][0].getType();
+    std::vector<TypeNode> argTypes = tn.getArgTypes();
+    std::vector< Node > skolems;
+    NodeManager* nm = NodeManager::currentNM();
+    for (unsigned i = 0, nargs = argTypes.size(); i < nargs; i++)
+    {
+      Node k =
+          nm->mkSkolem("k", argTypes[i], "skolem created for extensionality.");
+      skolems.push_back( k );
+    }
+    Node t[2];
+    for (unsigned i = 0; i < 2; i++)
+    {
+      std::vector< Node > children;
+      Node curr = deq[0][i];
+      while( curr.getKind()==kind::HO_APPLY ){
+        children.push_back( curr[1] );
+        curr = curr[0];
+      }
+      children.push_back( curr );
+      std::reverse( children.begin(), children.end() );
+      children.insert( children.end(), skolems.begin(), skolems.end() );
+      t[i] = nm->mkNode(kind::APPLY_UF, children);
+    }
+    Node conc = t[0].eqNode( t[1] ).negate();
+    d_extensionality_deq[deq] = conc;
+    return conc;
+  }
+  return it->second;
+}
+
+unsigned TheoryUF::applyExtensionality(TNode deq)
+{
+  Assert(deq.getKind() == kind::NOT && deq[0].getKind() == kind::EQUAL);
+  Assert(deq[0][0].getType().isFunction());
+  // apply extensionality
+  if (d_extensionality.find(deq) == d_extensionality.end())
+  {
+    d_extensionality.insert(deq);
+    Node conc = getExtensionalityDeq(deq);
+    Node lem = NodeManager::currentNM()->mkNode( kind::OR, deq[0], conc );
+    Trace("uf-ho-lemma") << "uf-ho-lemma : extensionality : " << lem << std::endl;
+    d_out->lemma( lem );
+    return 1;
+  }
+  return 0;
+}
+
+unsigned TheoryUF::checkExtensionality(TheoryModel* m)
+{
+  unsigned num_lemmas = 0;
+  bool isCollectModel = (m != nullptr);
+  Trace("uf-ho") << "TheoryUF::checkExtensionality, collectModel="
+                 << isCollectModel << "..." << std::endl;
+  std::map< TypeNode, std::vector< Node > > func_eqcs;
+  eq::EqClassesIterator eqcs_i = eq::EqClassesIterator( &d_equalityEngine );
+  while( !eqcs_i.isFinished() ){
+    Node eqc = (*eqcs_i);
+    TypeNode tn = eqc.getType();
+    if( tn.isFunction() ){
+      // if during collect model, must have an infinite type
+      // if not during collect model, must have a finite type
+      if (tn.isInterpretedFinite() != isCollectModel)
+      {
+        func_eqcs[tn].push_back(eqc);
+        Trace("uf-ho-debug")
+            << "  func eqc : " << tn << " : " << eqc << std::endl;
+      }
+    }
+    ++eqcs_i;
+  }
+  
+  for( std::map< TypeNode, std::vector< Node > >::iterator itf = func_eqcs.begin(); 
+       itf != func_eqcs.end(); ++itf ){
+    for( unsigned j=0; j<itf->second.size(); j++ ){
+      for( unsigned k=(j+1); k<itf->second.size(); k++ ){ 
+        // if these equivalence classes are not explicitly disequal, do extensionality to ensure distinctness
+        if (!d_equalityEngine.areDisequal(
+                itf->second[j], itf->second[k], false))
+        {
+          Node deq = Rewriter::rewrite( itf->second[j].eqNode( itf->second[k] ).negate() );
+          // either add to model, or add lemma
+          if (isCollectModel)
+          {
+            // add extentionality disequality to the model
+            Node edeq = getExtensionalityDeq(deq);
+            Assert(edeq.getKind() == kind::NOT
+                   && edeq[0].getKind() == kind::EQUAL);
+            // introducing terms, must add required constraints, e.g. to
+            // force equalities between APPLY_UF and HO_APPLY terms
+            for (unsigned r = 0; r < 2; r++)
+            {
+              if (!collectModelInfoHoTerm(edeq[0][r], m))
+              {
+                return 1;
+              }
+            }
+            Trace("uf-ho-debug")
+                << "Add extensionality deq to model : " << edeq << std::endl;
+            if (!m->assertEquality(edeq[0][0], edeq[0][1], false))
+            {
+              return 1;
+            }
+          }
+          else
+          {
+            // apply extensionality lemma
+            num_lemmas += applyExtensionality(deq);
+          }
+        }
+      }   
+    }
+  }
+  return num_lemmas;
+}
+
+unsigned TheoryUF::applyAppCompletion( TNode n ) {
+  Assert( n.getKind()==kind::APPLY_UF );
+
+  //must expand into APPLY_HO version if not there already
+  Node ret = TheoryUfRewriter::getHoApplyForApplyUf( n );
+  if( !d_equalityEngine.hasTerm( ret ) || !d_equalityEngine.areEqual( ret, n ) ){
+    Node eq = ret.eqNode( n );
+    Trace("uf-ho-lemma") << "uf-ho-lemma : infer, by apply-expand : " << eq << std::endl;
+    d_equalityEngine.assertEquality(eq, true, d_true);
+    return 1;
+  }else{
+    Trace("uf-ho-debug") << "    ...already have " << ret << " == " << n << "." << std::endl;
+    return 0;
+  }
+}
+
+unsigned TheoryUF::checkAppCompletion() {
+  Trace("uf-ho") << "TheoryUF::checkApplyCompletion..." << std::endl;
+  // compute the operators that are relevant (those for which an HO_APPLY exist)
+  std::set< TNode > rlvOp;
+  eq::EqClassesIterator eqcs_i = eq::EqClassesIterator( &d_equalityEngine );
+  std::map< TNode, std::vector< Node > > apply_uf;
+  while( !eqcs_i.isFinished() ){
+    Node eqc = (*eqcs_i);
+    Trace("uf-ho-debug") << "  apply completion : visit eqc " << eqc << std::endl;
+    eq::EqClassIterator eqc_i = eq::EqClassIterator( eqc, &d_equalityEngine );
+    while( !eqc_i.isFinished() ){
+      Node n = *eqc_i;
+      if( n.getKind()==kind::APPLY_UF || n.getKind()==kind::HO_APPLY ){
+        int curr_sum = 0;
+        std::map< TNode, bool > curr_rops;
+        if( n.getKind()==kind::APPLY_UF ){
+          TNode rop = d_equalityEngine.getRepresentative( n.getOperator() );
+          if( rlvOp.find( rop )!=rlvOp.end() ){
+            // try if its operator is relevant
+            curr_sum = applyAppCompletion( n );
+            if( curr_sum>0 ){
+              return curr_sum;
+            }
+          }else{
+            // add to pending list
+            apply_uf[rop].push_back( n );
+          }
+          //arguments are also relevant operators  FIXME (github issue #1115)
+          for( unsigned k=0; k<n.getNumChildren(); k++ ){
+            if( n[k].getType().isFunction() ){
+              TNode rop = d_equalityEngine.getRepresentative( n[k] );
+              curr_rops[rop] = true;
+            }
+          }
+        }else{
+          Assert( n.getKind()==kind::HO_APPLY );
+          TNode rop = d_equalityEngine.getRepresentative( n[0] );
+          curr_rops[rop] = true;
+        }
+        for( std::map< TNode, bool >::iterator itc = curr_rops.begin(); itc != curr_rops.end(); ++itc ){
+          TNode rop = itc->first;
+          if( rlvOp.find( rop )==rlvOp.end() ){
+            rlvOp.insert( rop );
+            // now, try each pending APPLY_UF for this operator
+            std::map< TNode, std::vector< Node > >::iterator itu = apply_uf.find( rop );
+            if( itu!=apply_uf.end() ){
+              for( unsigned j=0; j<itu->second.size(); j++ ){
+                curr_sum = applyAppCompletion( itu->second[j] );
+                if( curr_sum>0 ){
+                  return curr_sum;
+                }
+              }
+            }
+          }
+        }
+      }
+      ++eqc_i;
+    }
+    ++eqcs_i;
+  }
+  return 0;
+}
+
+unsigned TheoryUF::checkHigherOrder() {
+  Trace("uf-ho") << "TheoryUF::checkHigherOrder..." << std::endl;
+
+  // infer new facts based on apply completion until fixed point 
+  unsigned num_facts;
+  do{
+    num_facts = checkAppCompletion();
+    if( d_conflict ){
+      Trace("uf-ho") << "...conflict during app-completion." << std::endl;
+      return 1;  
+    }
+  }while( num_facts>0 );
+
+  if( options::ufHoExt() ){
+    unsigned num_lemmas = 0;
+
+    num_lemmas = checkExtensionality();
+    if( num_lemmas>0 ){
+      Trace("uf-ho") << "...extensionality returned " << num_lemmas << " lemmas." << std::endl;
+      return num_lemmas;
+    }
+  }
+
+  Trace("uf-ho") << "...finished check higher order." << std::endl;
+
+  return 0;
+}
 
 } /* namespace CVC4::theory::uf */
 } /* namespace CVC4::theory */
