@@ -2,9 +2,9 @@
 /*! \file valuation.cpp
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Dejan Jovanovic, Andrew Reynolds
+ **   Dejan Jovanovic, Morgan Deters, Andrew Reynolds
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -22,6 +22,26 @@
 namespace CVC4 {
 namespace theory {
 
+std::ostream& operator<<(std::ostream& os, EqualityStatus s)
+{
+  switch (s)
+  {
+    case EQUALITY_TRUE_AND_PROPAGATED:
+      os << "EQUALITY_TRUE_AND_PROPAGATED";
+      break;
+    case EQUALITY_FALSE_AND_PROPAGATED:
+      os << "EQUALITY_FALSE_AND_PROPAGATED";
+      break;
+    case EQUALITY_TRUE: os << "EQUALITY_TRUE"; break;
+    case EQUALITY_FALSE: os << "EQUALITY_FALSE"; break;
+    case EQUALITY_TRUE_IN_MODEL: os << "EQUALITY_TRUE_IN_MODEL"; break;
+    case EQUALITY_FALSE_IN_MODEL: os << "EQUALITY_FALSE_IN_MODEL"; break;
+    case EQUALITY_UNKNOWN: os << "EQUALITY_UNKNOWN"; break;
+    default: Unhandled(); break;
+  }
+  return os;
+}
+
 bool equalityStatusCompatible(EqualityStatus s1, EqualityStatus s2) {
   switch (s1) {
   case EQUALITY_TRUE:
