@@ -2,9 +2,9 @@
 /*! \file gmp_util.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Dejan Jovanovic, Paul Meng
+ **   Dejan Jovanovic, Morgan Deters, Andres Noetzli
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -20,6 +20,13 @@
 #ifndef __CVC4__GMP_UTIL_H
 #define __CVC4__GMP_UTIL_H
 
+/*
+ * Older versions of GMP in combination with newer versions of GCC and C++11
+ * cause errors: https://gcc.gnu.org/gcc-4.9/porting_to.html
+ * Including <cstddef> is a workaround for this issue.
+ */
+#include <cstddef>
+
 #include <gmpxx.h>
 
 namespace CVC4 {
