@@ -2,9 +2,9 @@
 /*! \file hash.h
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Christopher L. Conway, Paul Meng
+ **   Morgan Deters, Andres Noetzli, Christopher L. Conway
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2018 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -20,13 +20,11 @@
 #ifndef __CVC4__HASH_H
 #define __CVC4__HASH_H
 
-// in case it's not been declared as a namespace yet
-namespace __gnu_cxx {}
+#include <cstdint>
+#include <functional>
+#include <string>
 
-#include <ext/hash_map>
-#include <ext/hash_set>
-
-namespace __gnu_cxx {
+namespace std {
 
 #ifdef CVC4_NEED_HASH_UINT64_T
 // on some versions and architectures of GNU C++, we need a
@@ -39,23 +37,31 @@ struct hash<uint64_t> {
 };/* struct hash<uint64_t> */
 #endif /* CVC4_NEED_HASH_UINT64_T */
 
-}/* __gnu_cxx namespace */
-
-// hackish: treat hash stuff as if it were in std namespace
-namespace std { using namespace __gnu_cxx; }
+}/* std namespace */
 
 namespace CVC4 {
 
-struct StringHashFunction {
-  size_t operator()(const std::string& str) const {
-    return __gnu_cxx::hash<const char*>()(str.c_str());
-  }
-};/* struct StringHashFunction */
+namespace fnv1a {
+
+/**
+ * FNV-1a hash algorithm for 64-bit numbers.
+ *
+ * More details here: http://www.isthe.com/chongo/tech/comp/fnv/index.html
+ */
+inline uint64_t fnv1a_64(uint64_t v, uint64_t hash = 14695981039346656037U) {
+  hash ^= v;
+  // Compute (hash * 1099511628211)
+  return hash + (hash << 1) + (hash << 4) + (hash << 5) + (hash << 7) +
+         (hash << 8) + (hash << 40);
+}
+
+}  // namespace fnv1a
 
 template <class T, class U, class HashT = std::hash<T>, class HashU = std::hash<U> >
 struct PairHashFunction {
   size_t operator()(const std::pair<T, U>& pr) const {
-    return HashT()(pr.first) ^ HashU()(pr.second);
+    uint64_t hash = fnv1a::fnv1a_64(HashT()(pr.first));
+    return static_cast<size_t>(fnv1a::fnv1a_64(HashU()(pr.second), hash));
   }
 };/* struct PairHashFunction */
 
